@@ -48,10 +48,7 @@ public:
     NodePtr childFromTag (const QString & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "ASX"; }
     bool expose () const { return !pretty_name.isEmpty (); }
-    /**
-     * True if no mrl children
-     */
-    bool isPlayable ();
+    PlayType playType ();
 };
 
 /**
@@ -66,7 +63,7 @@ public:
     /**
      * False, but since we might have a 'base' child, we can have a rel. src
      */
-    bool isPlayable ();
+    PlayType playType ();
     void activate ();
     void deactivate ();
     bool expose () const;
