@@ -122,7 +122,7 @@ public:
         Feat_Viewer = 0x01, Feat_Controls = 0x02,
         Feat_Label = 0x04, Feat_StatusBar = 0x08,
         Feat_InfoPanel = 0x10, Feat_VolumeSlider = 0x20, Feat_PlayList = 0x40,
-        Feat_All = 0xff
+        Feat_ImageWindow = 0x80, Feat_All = 0xff
     };
     KMPlayerPart (QWidget * wparent, const char * wname,
               QObject * parent, const char * name, const QStringList &args);
