@@ -17,14 +17,12 @@
  **/
 
 #include <config.h>
+
 #include <qcolor.h>
-#include <qpainter.h>
-#include <qpixmap.h>
 #include <qimage.h>
 #include <qtimer.h>
 
 #include <kdebug.h>
-#include <kurl.h>
 
 #include "kmplayer_rp.h"
 #include "kmplayer_smil.h"
@@ -34,14 +32,11 @@ using namespace KMPlayer;
 
 KDE_NO_CDTOR_EXPORT RP::Imfl::Imfl (NodePtr & d)
   : Mrl (d, id_node_imfl),
-    x (0), y (0), w (0), h (0),
     fit (fit_hidden),
-    width (0), height (0), duration (0),
-    image (0L), cached_image (0L) {}
+    duration (0),
+    needs_scene_img (0) {}
 
 KDE_NO_CDTOR_EXPORT RP::Imfl::~Imfl () {
-    delete image;
-    delete cached_image;
 }
 
 KDE_NO_EXPORT void RP::Imfl::defer () {
@@ -56,6 +51,11 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
     kdDebug () << "RP::Imfl::activate " << endl;
     resolved = true;
     setState (state_activated);
+    surface = Mrl::getSurface (this);
+    if (!surface) {
+        finish ();
+        return;
+    }
     int timings_count = 0;
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
         switch (n->id) {
@@ -64,6 +64,7 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
             case RP::id_node_fadeout:
             case RP::id_node_fill:
             case RP::id_node_wipe:
+            case RP::id_node_viewchange:
                 n->activate (); // set their start timers
                 timings_count++;
                 break;
@@ -77,21 +78,21 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
                     } else if (!strcmp (a->nodeName (), "height")) {
                         height = a->nodeValue ().toInt ();
                     } else if (!strcmp (a->nodeName (), "duration")) {
-                        parseTime (a->nodeValue ().lower (), duration);
+                        int dur;
+                        parseTime (a->nodeValue ().lower (), dur);
+                        duration = dur;
                     }
                 }
                 break;
         }
     if (width <= 0 || width > 32000)
-        width = w;
+        width = surface->bounds.width ();
     if (height <= 0 || height > 32000)
-        height = h;
+        height = surface->bounds.height ();
     if (width > 0 && height > 0) {
-        image = new QPixmap (width, height);
-        image->fill ();
+        surface->xscale = 1.0 * surface->bounds.width () / width;
+        surface->yscale = 1.0 * surface->bounds.height () / height;
     }
-    if (parentNode ())
-        parentNode ()->registerEventHandler (this);
     if (duration > 0)
         duration_timer = document ()->setTimeout (this, duration * 100);
     else if (!timings_count)
@@ -135,50 +136,21 @@ KDE_NO_EXPORT void RP::Imfl::deactivate () {
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
         if (n->active ())
             n->deactivate ();
-    delete image;
-    image = 0L;
-    invalidateCachedImage ();
-    if (parentNode ())
-        parentNode ()->deregisterEventHandler (this);
-}
-
-KDE_NO_EXPORT void RP::Imfl::invalidateCachedImage () {
-    delete cached_image;
-    cached_image = 0L;
+    surface = Mrl::getSurface (0L);
 }
 
 KDE_NO_EXPORT bool RP::Imfl::handleEvent (EventPtr event) {
     if (event->id () == event_sized) {
-        SizeEvent * e = static_cast <SizeEvent *> (event.ptr ());
-        x = e->x ();
-        y = e->y ();
-        w = e->w ();
-        h = e->h ();
-        fit = e->fit;
-        matrix = e->matrix;
-        //kdDebug () << "RP::Imfl sized: " << x << "," << y << " " << w << "x" << h << endl;
-    } else if (event->id () == event_paint) {
-        if (active () && image) {
-            PaintEvent * p = static_cast <PaintEvent *> (event.ptr ());
-            //kdDebug () << "RP::Imfl paint: " << x << "," << y << " " << w << "x" << h << endl;
-            if (w == width && h == height) {
-                p->painter.drawPixmap (x, y, *image);
-            } else {
-                int x1=0, y1=0, w1=width, h1=height;
-                if (fit == fit_fill) {
-                    w1 = w;
-                    h1 = h;
-                } else
-                    matrix.getXYWH (x1, y1, w1, h1);
-                if (!cached_image ||
-                        cached_image->width () != w1 ||
-                        cached_image->height () != h1) {
-                    delete cached_image;
-                    QImage img;
-                    img = *image;
-                    cached_image = new QPixmap (img.scale (w1, h1));
-                }
-                p->painter.drawPixmap (x, y, *cached_image);
+        fit = static_cast <SizeEvent *> (event.ptr ())->fit;
+        if (surface) {
+            if (fit == fit_fill) {
+                surface->xscale = 1.0 * surface->bounds.width () / width;
+                surface->yscale = 1.0 * surface->bounds.height () / height;
+            } else { // keep aspect, center (and so add offset) ??
+                if (surface->xscale > surface->yscale)
+                    surface->xscale = surface->yscale;
+                else
+                    surface->yscale = surface->xscale;
             }
         }
     } else if (event->id () == event_timer) {
@@ -193,6 +165,10 @@ KDE_NO_EXPORT bool RP::Imfl::handleEvent (EventPtr event) {
     return true;
 }
 
+KDE_NO_EXPORT void RP::Imfl::accept (Visitor * v) {
+    v->visit (this);
+}
+
 KDE_NO_EXPORT NodePtr RP::Imfl::childFromTag (const QString & tag) {
     const char * ctag = tag.latin1 ();
     if (!strcmp (ctag, "head"))
@@ -203,6 +179,8 @@ KDE_NO_EXPORT NodePtr RP::Imfl::childFromTag (const QString & tag) {
         return new RP::Fill (m_doc);
     else if (!strcmp (ctag, "wipe"))
         return new RP::Wipe (m_doc);
+    else if (!strcmp (ctag, "viewchange"))
+        return new RP::ViewChange (m_doc);
     else if (!strcmp (ctag, "crossfade"))
         return new RP::Crossfade (m_doc);
     else if (!strcmp (ctag, "fadein"))
@@ -213,18 +191,17 @@ KDE_NO_EXPORT NodePtr RP::Imfl::childFromTag (const QString & tag) {
 }
 
 KDE_NO_EXPORT void RP::Imfl::repaint () {
-    PlayListNotify * n = document()->notify_listener;
     if (!active ())
         kdWarning () << "Spurious Imfl repaint" << endl;
-    else if (n && w > 0 && h > 0)
-        n->repaintRect (x, y, w, h);
+    else if (surface && width > 0 && height > 0)
+        surface->repaint (0, 0, width, height);
 }
 
 KDE_NO_CDTOR_EXPORT RP::Image::Image (NodePtr & doc)
- : Mrl (doc, id_node_image), image (0L) {}
+ : Mrl (doc, id_node_image)
+{}
 
 KDE_NO_CDTOR_EXPORT RP::Image::~Image () {
-    delete image;
 }
 
 KDE_NO_EXPORT void RP::Image::closed () {
@@ -235,10 +212,13 @@ KDE_NO_EXPORT void RP::Image::activate () {
     kdDebug () << "RP::Image::activate" << endl;
     setState (state_activated);
     isPlayable (); // update src attribute
-    wget (absolutePath ());
+    cached_img.setUrl (absolutePath ());
+    if (cached_img.data->isEmpty ())
+        wget (absolutePath ());
 }
 
 KDE_NO_EXPORT void RP::Image::deactivate () {
+    cached_img.setUrl (QString ());
     setState (state_deactivated);
     postpone_lock = 0L;
 }
@@ -246,16 +226,14 @@ KDE_NO_EXPORT void RP::Image::deactivate () {
 
 KDE_NO_EXPORT void RP::Image::remoteReady (QByteArray & data) {
     kdDebug () << "RP::Image::remoteReady" << endl;
-    if (!data.isEmpty ()) {
+    if (!data.isEmpty () && cached_img.data->isEmpty ()) {
         QImage * img = new QImage (data);
-        if (!img->isNull ()) {
-            image = img;
-            image->setAlphaBuffer (true);
-        } else
+        if (!img->isNull ())
+            cached_img.data->image = img;
+        else
             delete img;
     }
     postpone_lock = 0L;
-    kdDebug () << "RP::Image::remoteReady " << (void *) image << endl;
 }
 
 KDE_NO_EXPORT bool RP::Image::isReady (bool postpone_if_not) {
@@ -265,16 +243,22 @@ KDE_NO_EXPORT bool RP::Image::isReady (bool postpone_if_not) {
 }
 
 KDE_NO_CDTOR_EXPORT RP::TimingsBase::TimingsBase (NodePtr & d, const short i)
- : Element (d, i), start (0), duration (0), x (0), y (0), w (0), h (0) {}
+ : Element (d, i), x (0), y (0), w (0), h (0), start (0), duration (0) {}
 
 KDE_NO_EXPORT void RP::TimingsBase::activate () {
     setState (state_activated);
+    x = y = w = h = 0;
+    srcx = srcy = srcw = srch = 0;
     for (Attribute * a= attributes ()->first ().ptr (); a; a = a->nextSibling ().ptr ()) {
-        if (!strcasecmp (a->nodeName (), "start"))
-            parseTime (a->nodeValue ().lower (), start);
-        else if (!strcasecmp (a->nodeName (), "duration"))
-            parseTime (a->nodeValue ().lower (), duration);
-        else if (!strcasecmp (a->nodeName (), "target")) {
+        if (!strcasecmp (a->nodeName (), "start")) {
+            int dur;
+            parseTime (a->nodeValue ().lower (), dur);
+            start = dur;
+        } else if (!strcasecmp (a->nodeName (), "duration")) {
+            int dur;
+            parseTime (a->nodeValue ().lower (), dur);
+            duration = dur;
+        } else if (!strcasecmp (a->nodeName (), "target")) {
             for (NodePtr n = parentNode()->firstChild(); n; n= n->nextSibling())
                 if (convertNode <Element> (n)->getAttribute ("handle") == a->nodeValue ())
                     target = n;
@@ -286,6 +270,14 @@ KDE_NO_EXPORT void RP::TimingsBase::activate () {
             w = a->nodeValue ().toInt ();
         } else if (!strcasecmp (a->nodeName (), "dsth")) {
             h = a->nodeValue ().toInt ();
+        } else if (!strcasecmp (a->nodeName (), "srcx")) {
+            srcx = a->nodeValue ().toInt ();
+        } else if (!strcasecmp (a->nodeName (), "srcy")) {
+            srcy = a->nodeValue ().toInt ();
+        } else if (!strcasecmp (a->nodeName (), "srcw")) {
+            srcw = a->nodeValue ().toInt ();
+        } else if (!strcasecmp (a->nodeName (), "srch")) {
+            srch = a->nodeValue ().toInt ();
         }
     }
     start_timer = document ()->setTimeout (this, start *100);
@@ -324,6 +316,7 @@ KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (EventPtr event) {
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::begin () {
+    progress = 0;
     setState (state_began);
     if (target)
         target->begin ();
@@ -334,10 +327,15 @@ KDE_NO_EXPORT void RP::TimingsBase::begin () {
     }
 }
 
-KDE_NO_EXPORT void RP::TimingsBase::update (int /*percentage*/) {
+KDE_NO_EXPORT void RP::TimingsBase::update (int percentage) {
+    progress = percentage;
+    Node * p = parentNode ().ptr ();
+    if (p->id == RP::id_node_imfl)
+        static_cast <RP::Imfl *> (p)->repaint ();
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::finish () {
+    progress = 100;
     if (start_timer) {
         document ()->cancelTimer (start_timer);
         start_timer = 0;
@@ -358,7 +356,7 @@ KDE_NO_EXPORT void RP::Crossfade::activate () {
 }
 
 KDE_NO_EXPORT void RP::Crossfade::begin () {
-    kdDebug () << "RP::Crossfade::begin" << endl;
+    //kdDebug () << "RP::Crossfade::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
@@ -369,41 +367,18 @@ KDE_NO_EXPORT void RP::Crossfade::begin () {
     }
 }
 
-KDE_NO_EXPORT void RP::Crossfade::update (int percentage) {
-    if (percentage > 0 && percentage < 100)
-        return; // we do no crossfading yet, just paint the first and last time
-    Node * p = parentNode ().ptr ();
-    if (p->id != RP::id_node_imfl) {
-        kdWarning () << "crossfade update: no imfl parent found" << endl;
-        return;
-    }
-    RP::Imfl * imfl = static_cast <RP::Imfl *> (p);
-    if (imfl->image && target && target->id == id_node_image) {
-        RP::Image * img = static_cast <RP::Image *> (target.ptr ());
-        if (img->image) {
-            QPainter painter;
-            painter.begin (imfl->image);
-            painter.drawImage (x, y, *img->image);
-            painter.end ();
-            imfl->invalidateCachedImage ();
-            imfl->repaint ();
-        }
-    }
+KDE_NO_EXPORT void RP::Crossfade::accept (Visitor * v) {
+    v->visit (this);
 }
 
 KDE_NO_EXPORT void RP::Fadein::activate () {
     // pickup color from Fill that should be declared before this node
     from_color = 0;
-    for (NodePtr n = previousSibling (); n; n = n->previousSibling ())
-        if (n->id == id_node_fill) {
-            from_color = convertNode <RP::Fill> (n)->fillColor ();
-            break;
-        }
     TimingsBase::activate ();
 }
 
 KDE_NO_EXPORT void RP::Fadein::begin () {
-    kdDebug () << "RP::Fadein::begin" << endl;
+    //kdDebug () << "RP::Fadein::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
@@ -414,33 +389,8 @@ KDE_NO_EXPORT void RP::Fadein::begin () {
     }
 }
 
-KDE_NO_EXPORT void RP::Fadein::update (int percentage) {
-    Node * p = parentNode ().ptr ();
-    if (p->id != RP::id_node_imfl) {
-        kdWarning () << "fadein begin: no imfl parent found" << endl;
-        return;
-    }
-    RP::Imfl * imfl = static_cast <RP::Imfl *> (p);
-    if (imfl->image && target && target->id == id_node_image) {
-        RP::Image * img = static_cast <RP::Image *> (target.ptr ());
-        if (img->image) {
-            QPainter painter;
-            painter.begin (imfl->image);
-            painter.drawImage (x, y, *img->image);
-            //QImage alpha (img->image->width(), img->image->height(), img->image->depth ());
-            //alpha.setAlphaBuffer (true);
-            //alpha.fill (((0x00ff * (100 - percentage) / 100) << 24) | (0xffffff & from_color));
-            //painter.fillRect (x, y, img->image->width(), img->image->height(), QBrush (QColor (((0x00ff * (100 - percentage) / 100) << 24) | (0xffffff & from_color)), Qt::Dense7Pattern));
-            if (percentage < 90) {
-                int brush_pat = ((int) Qt::SolidPattern) + 10 * percentage / 125;
-                painter.fillRect (x, y, img->image->width(), img->image->height(), QBrush (QColor (from_color), (Qt::BrushStyle) brush_pat));
-            }
-            //painter.drawImage (x, y, alpha, Qt::OrderedAlphaDither);
-            painter.end ();
-            imfl->invalidateCachedImage ();
-            imfl->repaint ();
-        }
-    }
+KDE_NO_EXPORT void RP::Fadein::accept (Visitor * v) {
+    v->visit (this);
 }
 
 KDE_NO_EXPORT void RP::Fadeout::activate () {
@@ -449,30 +399,12 @@ KDE_NO_EXPORT void RP::Fadeout::activate () {
 }
 
 KDE_NO_EXPORT void RP::Fadeout::begin () {
-    kdDebug () << "RP::Fadeout::begin" << endl;
+    //kdDebug () << "RP::Fadeout::begin" << endl;
     TimingsBase::begin ();
 }
 
-KDE_NO_EXPORT void RP::Fadeout::update (int percentage) {
-    Node * p = parentNode ().ptr ();
-    if (p->id == RP::id_node_imfl) {
-        RP::Imfl * imfl = static_cast <RP::Imfl *> (p);
-        if (imfl->image) {
-            int brush_pat = ((int) Qt::Dense7Pattern) - 10 * percentage / 126;
-            int pw = w;
-            int ph = h;
-            if (!w || !h) {
-                pw = imfl->image->width ();
-                ph = imfl->image->height ();
-            }
-            QPainter painter;
-            painter.begin (imfl->image);
-            painter.fillRect (x, y, pw, ph, QBrush (QColor (to_color), (Qt::BrushStyle) brush_pat));
-            painter.end ();
-            imfl->invalidateCachedImage ();
-            imfl->repaint ();
-        }
-    }
+KDE_NO_EXPORT void RP::Fadeout::accept (Visitor * v) {
+    v->visit (this);
 }
 
 KDE_NO_EXPORT void RP::Fill::activate () {
@@ -481,23 +413,12 @@ KDE_NO_EXPORT void RP::Fill::activate () {
 }
 
 KDE_NO_EXPORT void RP::Fill::begin () {
-    TimingsBase::begin ();
-    Node * p = parentNode ().ptr ();
-    if (p->id == RP::id_node_imfl) {
-        RP::Imfl * imfl = static_cast <RP::Imfl *> (p);
-        if (imfl->image) {
-            if (!w || !h) {
-                imfl->image->fill (color);
-            } else {
-                QPainter painter;
-                painter.begin (imfl->image);
-                painter.fillRect (x, y, w, h, QBrush (color));
-                painter.end ();
-            }
-            imfl->invalidateCachedImage ();
-            imfl->repaint ();
-        }
-    }
+    setState (state_began);
+    update (0);
+}
+
+KDE_NO_EXPORT void RP::Fill::accept (Visitor * v) {
+    v->visit (this);
 }
 
 KDE_NO_EXPORT void RP::Wipe::activate () {
@@ -514,7 +435,7 @@ KDE_NO_EXPORT void RP::Wipe::activate () {
 }
 
 KDE_NO_EXPORT void RP::Wipe::begin () {
-    kdDebug () << "RP::Wipe::begin" << endl;
+    //kdDebug () << "RP::Wipe::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
@@ -525,43 +446,30 @@ KDE_NO_EXPORT void RP::Wipe::begin () {
     }
 }
 
-KDE_NO_EXPORT void RP::Wipe::update (int percentage) {
+KDE_NO_EXPORT void RP::Wipe::accept (Visitor * v) {
+    v->visit (this);
+}
+
+KDE_NO_EXPORT void RP::ViewChange::activate () {
+    TimingsBase::activate ();
+}
+
+KDE_NO_EXPORT void RP::ViewChange::begin () {
+    kdDebug () << "RP::ViewChange::begin" << endl;
+    setState (state_began);
     Node * p = parentNode ().ptr ();
-    if (p->id != RP::id_node_imfl) {
-        kdWarning () << "wipe update: no imfl parent found" << endl;
-        return;
-    }
-    RP::Imfl * imfl = static_cast <RP::Imfl *> (p);
-    if (imfl->image && target && target->id == id_node_image) {
-        RP::Image * img = static_cast <RP::Image *> (target.ptr ());
-        if (img->image) {
-            QPainter painter;
-            painter.begin (imfl->image);
-            int dx = x, dy = y;
-            int sx = 0, sy = 0;
-            int sw = img->image->width ();
-            int sh = img->image->height ();
-            if (direction == dir_right) {
-                int iw = sw * percentage / 100;
-                sx = sw - iw;
-                sw = iw;
-            } else if (direction == dir_left) {
-                int iw = sw * percentage / 100;
-                dx += sw - iw;
-                sw = iw;
-            } else if (direction == dir_down) {
-                int ih = sh * percentage / 100;
-                sy = sh - ih;
-                sh = ih;
-            } else if (direction == dir_up) {
-                int ih = sh * percentage / 100;
-                dy += sh - ih;
-                sh = ih;
-            }
-            painter.drawImage (dx, dy, *img->image, sx, sy, sw, sh);
-            painter.end ();
-            imfl->invalidateCachedImage ();
-            imfl->repaint ();
-        }
-    }
+    if (p->id == RP::id_node_imfl)
+        static_cast <RP::Imfl *> (p)->needs_scene_img++;
+    update (0);
+}
+
+KDE_NO_EXPORT void RP::ViewChange::finish () {
+    Node * p = parentNode ().ptr ();
+    if (p && p->id == RP::id_node_imfl)
+        static_cast <RP::Imfl *> (p)->needs_scene_img--;
+    TimingsBase::finish ();
+}
+
+KDE_NO_EXPORT void RP::ViewChange::accept (Visitor * v) {
+    v->visit (this);
 }
