@@ -1,4 +1,4 @@
-/* This file is part of the KMPlayer project
+/* This file is part of the KDE project
  *
  * Copyright (C) 2006-2007 Koos Vriezen <koos.vriezen@gmail.com>
  *
@@ -21,8 +21,15 @@
 #ifndef _KMPLAYER_RP_H_
 #define _KMPLAYER_RP_H_
 
+#include <qobject.h>
+#include <qstring.h>
+
 #include "kmplayerplaylist.h"
-#include "surface.h"
+#include "kmplayer_smil.h"
+
+namespace KIO {
+    class Job;
+}
 
 namespace KMPlayer {
 
@@ -41,30 +48,28 @@ const short id_node_fadein = 156;
 const short id_node_fadeout = 157;
 const short id_node_viewchange = 158;
 const short id_node_animate = 159;
-const short id_node_first = id_node_imfl;
-const short id_node_last = 160;
 
-class KMPLAYER_NO_EXPORT Imfl : public Mrl {
+class Imfl : public Mrl {
 public:
     Imfl (NodePtr & d);
     ~Imfl ();
     KDE_NO_EXPORT virtual const char * nodeName () const { return "imfl"; }
-    virtual Node *childFromTag (const String & tag);
-    virtual void closed ();
+    virtual NodePtr childFromTag (const QString & tag);
     virtual void defer ();      // start loading the images if not yet done
     virtual void activate ();   // start timings, handle paint events
     virtual void finish ();     // end the timings
     virtual void deactivate (); // stop handling paint events
+    virtual void childDone (NodePtr child); // for if no duration_timer set
     KDE_NO_EXPORT virtual bool expose () const { return false; }
-    KDE_NO_EXPORT virtual PlayType playType () { return play_type_image; }
-    virtual void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT virtual PlayType playType () const { return play_type_image; }
+    virtual bool handleEvent (EventPtr event);
     virtual void accept (Visitor *);
-    Surface *surface ();
     void repaint (); // called whenever something changes on image
     Fit fit;        // how to layout images
+    Single width, height;     // cached attributes of head
     unsigned int duration; // cached attributes of head
-    Command *duration_timer;
-    SurfacePtrW rp_surface;
+    TimerInfoPtrW duration_timer;
+    SurfacePtr surface;
     int needs_scene_img;
 };
 
@@ -76,7 +81,7 @@ public:
     virtual void begin ();       // start_timer has expired
     virtual void finish ();      // ?duration_timer has expired?
     virtual void deactivate ();  // disabled
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual bool handleEvent (EventPtr event);
     KDE_NO_EXPORT virtual bool expose () const { return false; }
     int progress;
     Single x, y, w, h;
@@ -84,12 +89,11 @@ public:
     NodePtrW target;
 protected:
     void update (int percentage);
-    void cancelTimers ();
     unsigned int start, duration;
     int steps, curr_step;
-    Command *start_timer;
-    Command *duration_timer;
-    Command *update_timer;
+    TimerInfoPtrW start_timer;
+    TimerInfoPtrW duration_timer;
+    TimerInfoPtrW update_timer;
     ConnectionPtr document_postponed;
 };
 
@@ -161,23 +165,20 @@ public:
     virtual void accept (Visitor *);
 };
 
-class KMPLAYER_NO_EXPORT Image : public Mrl {
+class KMPLAYER_NO_EXPORT Image : public RemoteObject, public Mrl {
     PostponePtr postpone_lock;
 public:
     Image (NodePtr & d);
     ~Image ();
     KDE_NO_EXPORT virtual const char * nodeName () const { return "image"; }
     virtual void activate ();
-    virtual void begin ();
     virtual void deactivate ();
     virtual void closed ();
-    virtual void *message (MessageType msg, void *content=NULL);
     bool isReady (bool postpone_if_not = false); // is downloading ready
-    Surface *surface ();
-    SurfacePtrW img_surface;
-    bool expose () const { return false; }
+    CachedImage cached_img;
+    //bool expose () const { return false; }
 protected:
-    void dataArrived ();
+    virtual void remoteReady (QByteArray & data);
 };
 
 } // RP namespace
