@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  *
- * Copyright (C) 2005 Koos Vriezen <koos.vriezen@xs4all.nl>
+ * Copyright (C) 2005-2006 Koos Vriezen <koos.vriezen@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,8 @@
 #ifndef _KMPLAYER_RSS_H_
 #define _KMPLAYER_RSS_H_
 
+#include <qstring.h>
+
 #include "kmplayerplaylist.h"
 
 namespace KMPlayer {
@@ -33,24 +35,22 @@ const short id_node_item = 202;
 const short id_node_title = 203;
 const short id_node_description = 204;
 const short id_node_enclosure = 205;
-const short id_node_category = 206;
-const short id_node_ignored = 207;
 
 /**
  * '<RSS>' tag
  */
-class KMPLAYER_NO_EXPORT Rss : public Mrl {
+class Rss : public Mrl {
 public:
     KDE_NO_CDTOR_EXPORT Rss (NodePtr & d) : Mrl (d, id_node_rss) {}
-    Node *childFromTag (const String & tag);
+    NodePtr childFromTag (const QString & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "rss"; }
     bool expose () const { return false; }
 };
-
-class KMPLAYER_NO_EXPORT Channel : public Title {
+    
+class KMPLAYER_NO_EXPORT Channel : public Mrl {
 public:
-    KDE_NO_CDTOR_EXPORT Channel (NodePtr & d) : Title (d, id_node_channel) {}
-    Node *childFromTag (const String & tag);
+    KDE_NO_CDTOR_EXPORT Channel (NodePtr & d) : Mrl (d, id_node_channel) {}
+    NodePtr childFromTag (const QString & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "channel"; }
     PlayType playType () { return play_type_none; }
     void closed ();
@@ -59,19 +59,17 @@ public:
 
 class KMPLAYER_NO_EXPORT Item : public Mrl {
 public:
-    KDE_NO_CDTOR_EXPORT Item (NodePtr & d)
-        : Mrl (d, id_node_item), timer (NULL) {}
-    Node *childFromTag (const String & tag);
-    String description () const;
+    KDE_NO_CDTOR_EXPORT Item (NodePtr & d) : Mrl (d, id_node_item) {}
+    NodePtr childFromTag (const QString & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "item"; }
     PlayType playType () { return cached_play_type; }
     Mrl * linkNode ();
     void closed ();
     void activate ();
     void deactivate ();
-    void message (MessageType msg, void *content=NULL);
+    bool handleEvent (EventPtr event);
     NodePtrW enclosure;
-    Command *timer;
+    TimerInfoPtrW timer;
 };
 
 class KMPLAYER_NO_EXPORT Enclosure : public Mrl {
