@@ -63,6 +63,7 @@
 #include "kmplayer.h"
 #include "kmplayerview.h"
 #include "playlistview.h"
+#include "viewarea.h"
 #include "kmplayercontrolpanel.h"
 #include "kmplayerpartbase.h"
 #include "kmplayerprocess.h"
@@ -241,12 +242,12 @@ KDE_NO_CDTOR_EXPORT
 Recent::Recent (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QString &url) 
   : KMPlayer::Mrl (doc, id_node_recent_node), app (a) {
     src = url;
-    setAttribute ("url", url);
+    setAttribute (KMPlayer::StringPool::attr_url, url);
 }
 
 KDE_NO_EXPORT void Recent::closed () {
     if (src.isEmpty ())
-        src = getAttribute ("url");
+        src = getAttribute (KMPlayer::StringPool::attr_url);
 }
 
 KDE_NO_EXPORT void Recent::activate () {
@@ -258,7 +259,7 @@ Group::Group (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QString & pn)
   : KMPlayer::Mrl (doc, KMPlayer::id_node_group_node), app (a) {
     pretty_name = pn;
     if (!pn.isEmpty ())
-        setAttribute ("title", pn);
+        setAttribute (KMPlayer::StringPool::attr_title, pn);
 }
 
 KDE_NO_EXPORT KMPlayer::NodePtr Group::childFromTag (const QString & tag) {
@@ -271,7 +272,7 @@ KDE_NO_EXPORT KMPlayer::NodePtr Group::childFromTag (const QString & tag) {
 
 KDE_NO_EXPORT void Group::closed () {
     if (pretty_name.isEmpty ())
-        pretty_name = getAttribute ("title");
+        pretty_name = getAttribute (KMPlayer::StringPool::attr_title);
 }
 
 KDE_NO_EXPORT void Playlist::defer () {
@@ -319,12 +320,12 @@ KDE_NO_CDTOR_EXPORT
 PlaylistItem::PlaylistItem (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QString &url) 
   : KMPlayer::Mrl (doc, KMPlayer::id_node_playlist_item), app (a) {
     src = url;
-    setAttribute ("url", url);
+    setAttribute (KMPlayer::StringPool::attr_url, url);
 }
 
 KDE_NO_EXPORT void PlaylistItem::closed () {
     if (src.isEmpty ())
-        src = getAttribute ("url");
+        src = getAttribute (KMPlayer::StringPool::attr_url);
 }
 
 KDE_NO_EXPORT void PlaylistItem::activate () {
@@ -365,7 +366,7 @@ KDE_NO_EXPORT void PlaylistItem::activate () {
 
 KDE_NO_EXPORT void PlaylistItem::setNodeName (const QString & s) {
     src = s;
-    setAttribute ("url", s);
+    setAttribute (KMPlayer::StringPool::attr_url, s);
 }
 
 KDE_NO_CDTOR_EXPORT
@@ -373,7 +374,7 @@ PlaylistGroup::PlaylistGroup (KMPlayer::NodePtr & doc, KMPlayerApp * a, const QS
   : KMPlayer::Mrl (doc, KMPlayer::id_node_group_node), app (a), playmode (false) {
     pretty_name = pn;
     if (!pn.isEmpty ())
-        setAttribute ("title", pn);
+        setAttribute (KMPlayer::StringPool::attr_title, pn);
 }
 
 KDE_NO_CDTOR_EXPORT
@@ -393,12 +394,12 @@ KDE_NO_EXPORT KMPlayer::NodePtr PlaylistGroup::childFromTag (const QString & tag
 
 KDE_NO_EXPORT void PlaylistGroup::closed () {
     if (pretty_name.isEmpty ())
-        pretty_name = getAttribute ("title");
+        pretty_name = getAttribute (KMPlayer::StringPool::attr_title);
 }
 
 KDE_NO_EXPORT void PlaylistGroup::setNodeName (const QString & t) {
     pretty_name = t;
-    setAttribute ("title", t);
+    setAttribute (KMPlayer::StringPool::attr_title, t);
 }
 
 KDE_NO_CDTOR_EXPORT KMPlayerApp::KMPlayerApp(QWidget* , const char* name)
@@ -656,7 +657,7 @@ KDE_NO_EXPORT void KMPlayerApp::playerStarted () {
             }
         }
         if (!more && count > 10) {
-            more = new Group (recents, this, i18n ("More ..."));
+            more = new Group (recents, this, i18n ("More..."));
             recents->appendChild (more);
         }
         if (more) {
@@ -706,7 +707,7 @@ KDE_NO_EXPORT void KMPlayerApp::slotSourceChanged (KMPlayer::Source *olds, KMPla
 }
 
 KDE_NO_EXPORT void KMPlayerApp::dvdNav () {
-    slotStatusMsg(i18n("DVD Navigation ..."));
+    slotStatusMsg(i18n("DVD Navigation..."));
     m_player->setSource (m_player->sources () ["dvdnavsource"]);
     slotStatusMsg(i18n("Ready"));
 }
@@ -747,6 +748,7 @@ KDE_NO_EXPORT void KMPlayerApp::openVDR () {
         m_player->setSource (m_player->sources () ["vdrsource"]);
 }
 
+#ifdef HAVE_CAIRO
 struct IntroSource : public KMPlayer::Source {
     KMPlayerApp * m_app;
     IntroSource (KMPlayer::PartBase *p, KMPlayerApp * a)
@@ -771,27 +773,30 @@ KDE_NO_EXPORT void IntroSource::activate () {
         QTextStream ts (&file);
         KMPlayer::readXML (m_document, ts, QString::null, false);
     } else {
-        QString smil = QString::fromLatin1 ("<smil><head><layout>"
+        QString smil = QString::fromLatin1 (
+          "<smil><head><layout>"
           "<root-layout width='320' height='240' background-color='black'/>"
           "<region id='image1' left='31.25%' top='25%' width='37.5%' height='50%' z-order='1'/>"
           "<region id='reg1' top='10%' height='80%' z-order='2'>"
-          "<region id='image2' left='128' top='136' width='64' bottom='56'/>"
+          "<region id='image2' left='128' top='72' width='64' bottom='56'/>"
           "</region>"
-          "</layout></head><body><excl><seq>"
-          "<img src='%1' region='image1' dur='0.5s' fit='fill'/>"
+          "</layout>"
+          "<transition id='fadein-1' dur='1' type='fade'/>"
+          "</head>"
+          "<body>"
+          "<excl>"
           "<par>"
-          "<animate target='image1' attribute='width' from='37.5%' to='1%' dur='0.5' fill='freeze'/>"
-          "<animate target='image1' attribute='left' from='31.25%' to='50%' dur='0.5' fill='freeze'/>"
-          "<animate target='image1' attribute='height' from='50%' to='1%' dur='0.5' fill='freeze'/>"
-          "<animate target='image1' attribute='top' from='25%' to='50%' dur='0.5' fill='freeze'/>"
-          "<set target='image1' attribute='background-color' to='white' dur='0.5'/>"
+          "<img src='%1' region='image1' dur='1' fit='fill'/>"
+          "<animate target='image1' attribute='width' from='37.5%' to='1%' begin='.5' dur='0.5' fill='freeze'/>"
+          "<animate target='image1' attribute='left' from='31.25%' to='50%' begin='.5' dur='0.5' fill='freeze'/>"
+          "<animate target='image1' attribute='height' from='50%' to='1%' begin='.5' dur='0.5' fill='freeze'/>"
+          "<animate target='image1' attribute='top' from='25%' to='50%' begin='.5' dur='0.5' fill='freeze'/>"
+          "<img region='image2' src='%2' begin='0.5' dur='1' fit='hidden' fill='freeze' transIn='fadein-1'/>"
           "</par>"
-          "<par>"
-          "<animate target='reg1' attribute='background-color' calcMode='discrete' values='#000000;#000000;#020202;#060606;#0B0B0B;#111111;#191919;#222222;#2D2D2D;#393939;#464646;#555555;#656565;#777777;#8A8A8A;#9E9E9E;#B4B4B4;#CCCCCC;#E4E4E4;#FFFFFF' dur='0.6'/>"
-          "<animate target='image2' attribute='top' from='136' to='72' dur='0.4' fill='freeze'/>"
-          "<img src='%2' region='image2' dur='0.6' fit='hidden'/>"
-          "</par></seq><seq begin='reg1.activateEvent'/>"
-          "</excl></body></smil>").arg (locate ("data", "kmplayer/noise.gif")).arg (KGlobal::iconLoader()->iconPath (QString::fromLatin1 ("kmplayer"), -64));
+          "<seq begin='reg1.activateEvent'/>"
+          "</excl>"
+          "</body></smil>"
+          ).arg (locate ("data", "kmplayer/noise.gif")).arg (KGlobal::iconLoader()->iconPath (QString::fromLatin1 ("kmplayer"), -64));
         QTextStream ts (smil.utf8 (), IO_ReadOnly);
         KMPlayer::readXML (m_document, ts, QString::null, false);
     }
@@ -829,6 +834,7 @@ KDE_NO_EXPORT void IntroSource::deactivate () {
     if (!finished && m_document) // user opens a source while introducing
         m_document->reset ();
 }
+#endif
 
 KDE_NO_EXPORT void KMPlayerApp::restoreFromConfig () {
     if (m_player->view ()) {
@@ -849,9 +855,11 @@ KDE_NO_EXPORT void KMPlayerApp::openDocumentFile (const KURL& url)
             restoreFromConfig ();
             m_player->setSource (src);
             return;
+#ifdef HAVE_CAIRO
         } else if (!m_player->settings ()->no_intro && url.isEmpty ()) {
             m_player->setSource (new IntroSource (m_player, this));
             return;
+#endif
         } else {
             m_played_exit = true; // no intro, so no exit as well
             restoreFromConfig ();
@@ -1111,6 +1119,7 @@ KDE_NO_EXPORT void KMPlayerApp::slotMinimalMode () {
     minimalMode (true);
 }
 
+#ifdef HAVE_CAIRO
 struct ExitSource : public KMPlayer::Source {
     KDE_NO_CDTOR_EXPORT ExitSource (KMPlayer::PartBase *p)
         : KMPlayer::Source (i18n ("Exit"), p, "exitsource") {}
@@ -1137,7 +1146,7 @@ KDE_NO_EXPORT void ExitSource::activate () {
           "</region>"
           "</layout></head><body>"
           "<par>"
-          "<animate target='reg1' attribute='background-color' calcMode='discrete' values='#FFFFFF;#E4E4E4;#CCCCCC;#B4B4B4;#9E9E9E;#8A8A8A;#777777;#656565;#555555;#464646;#393939;#2D2D2D;#222222;#191919;#111111;#0B0B0B;#060606;#020202;#000000;#000000' dur='0.6'/>"
+          //"<animate target='reg1' attribute='background-color' calcMode='discrete' values='#FFFFFF;#E4E4E4;#CCCCCC;#B4B4B4;#9E9E9E;#8A8A8A;#777777;#656565;#555555;#464646;#393939;#2D2D2D;#222222;#191919;#111111;#0B0B0B;#060606;#020202;#000000;#000000' dur='0.6'/>"
           "<animate target='image' attribute='top' from='72' to='136' dur='0.4' fill='freeze'/>"
           "<img src='%2' region='image' dur='0.6' fit='hidden'/>"
           "</par>"
@@ -1166,6 +1175,7 @@ KDE_NO_EXPORT void ExitSource::stateElementChanged (KMPlayer::Node * node, KMPla
             m_player->view ())
        m_player->view ()->topLevelWidget ()->close ();
 }
+#endif
 
 KDE_NO_EXPORT bool KMPlayerApp::queryClose () {
     // KMPlayerVDRSource has to wait for pending commands like mute and quit
@@ -1177,15 +1187,19 @@ KDE_NO_EXPORT bool KMPlayerApp::queryClose () {
         QByteArray data, replydata;
         kapp->dcopClient ()->call (m_dcopName, "MainApplication-Interface", "quit()", data, replytype, replydata);
     }
-    if (m_played_exit || m_player->settings ()->no_intro)
+    if (m_played_exit || m_player->settings ()->no_intro || kapp->sessionSaving() )
         return true;
     if (m_auto_resize)
         disconnect(m_player, SIGNAL(sourceDimensionChanged()),this,SLOT(zoom100()));
     m_played_exit = true;
     if (!m_minimal_mode)
         minimalMode (false);
+#ifdef HAVE_CAIRO
     m_player->setSource (new ExitSource (m_player));
     return false;
+#else
+    return true;
+#endif
 }
 
 KDE_NO_EXPORT bool KMPlayerApp::queryExit()
