@@ -57,6 +57,7 @@
 #include "kmplayerpartbase.h"
 #include "kmplayerview.h"
 #include "playlistview.h"
+#include "viewarea.h"
 #include "kmplayercontrolpanel.h"
 #include "kmplayerconfig.h"
 #include "kmplayerprocess.h"
@@ -212,7 +213,7 @@ void PartBase::connectPanel (ControlPanel * panel) {
     panel->popupMenu ()->connectItem (ControlPanel::menu_video,
                                       m_view, SLOT(toggleVideoConsoleWindow()));
     panel->popupMenu ()->connectItem (ControlPanel::menu_playlist,
-                                      this, SLOT (showPlayListWindow ()));
+                                      m_view, SLOT (toggleShowPlaylist ()));
     connect (this, SIGNAL (statusUpdated (const QString &)),
              panel->view (), SLOT (setStatusMessage (const QString &)));
     //connect (panel (), SIGNAL (clicked ()), m_settings, SLOT (show ()));
@@ -879,7 +880,7 @@ KDE_NO_EXPORT void Source::setLanguages (const QStringList & alang, const QStrin
 
 void Source::setDimensions (NodePtr node, int w, int h) {
     Mrl * mrl = node ? node->mrl () : 0L;
-    if (mrl && mrl->view_mode == Mrl::Window) {
+    if (mrl && mrl->view_mode == Mrl::WindowMode) {
         mrl->width = w;
         mrl->height = h;
         float a = h > 0 ? 1.0 * w / h : 0.0;
@@ -907,11 +908,11 @@ void Source::setAspect (NodePtr node, float a) {
     Mrl * mrl = node ? node->mrl () : 0L;
     bool changed = false;
     if (mrl) {
-        if (mrl->view_mode == Mrl::Window)
+        if (mrl->view_mode == Mrl::WindowMode)
             changed |= (fabs (mrl->aspect - a) > 0.001);
         mrl->aspect = a;
     }
-    if (!mrl || mrl->view_mode == Mrl::Single) {
+    if (!mrl || mrl->view_mode == Mrl::SingleMode) {
         changed |= (fabs (m_aspect - a) > 0.001);
         m_aspect = a;
     }
@@ -1018,14 +1019,15 @@ void Source::playCurrent () {
     } else if (m_player->process ()->state () == Process::NotRunning) {
         m_player->process ()->ready (static_cast <View *> (m_player->view ())->viewer ());
     } else if (m_player->process ()) {
-        Mrl * mrl = m_current->mrl ();
-        if (mrl->view_mode == Mrl::Single) {
+        Mrl * mrl = m_back_request ? m_back_request->mrl () : m_current->mrl ();
+        if (mrl->view_mode == Mrl::SingleMode) {
             // don't reset the dimensions if we have any
             m_width = mrl->width;
             m_height = mrl->height;
             m_aspect = mrl->aspect;
         }
-        m_player->process ()->play (this, mrl->realMrl ());
+        m_back_request = 0L;
+        m_player->process ()->play (this, mrl->linkNode ());
     }
     //kdDebug () << "Source::playCurrent " << (m_current ? m_current->nodeName():" doc act:") <<  (m_document && !m_document->active ()) << " cur:" << (!m_current)  << " cur act:" << (m_current && !m_current->active ()) <<  endl;
     m_player->updateTree ();
@@ -1049,11 +1051,15 @@ static NodePtr findDepthFirst (NodePtr elm) {
 bool Source::requestPlayURL (NodePtr mrl) {
     //kdDebug() << "Source::requestPlayURL " << mrl->mrl ()->src << endl;
     if (m_player->process ()->state () > Process::Ready) {
+        if (m_player->process ()->mrl () == mrl->mrl ()->linkNode ())
+            return true;
         m_back_request = mrl; // still playing, schedule it
         m_player->process ()->stop ();
     } else {
-        if (mrl->mrl ()->view_mode == Mrl::Single)
+        if (mrl->mrl ()->view_mode == Mrl::SingleMode)
             m_current = mrl;
+        else
+            m_back_request = mrl;
         m_player->updateTree ();
         QTimer::singleShot (0, this, SLOT (playCurrent ()));
     }
@@ -1084,12 +1090,12 @@ bool Source::setCurrent (NodePtr mrl) {
 }
 
 void Source::stateElementChanged (Node * elm, Node::State os, Node::State ns) {
-    //kdDebug() << "[01;31mSource::stateElementChanged[00m " << elm->nodeName () << " state:" << (int) elm->state << " cur isPlayable:" << (m_current && m_current->isPlayable ()) << " elm==realMrl:" << (m_current && elm == m_current->mrl ()->realMrl ()) << " p state:" << m_player->process ()->state () << endl;
+    //kdDebug() << "[01;31mSource::stateElementChanged[00m " << elm->nodeName () << " state:" << (int) elm->state << " cur isPlayable:" << (m_current && m_current->isPlayable ()) << " elm==linkNode:" << (m_current && elm == m_current->mrl ()->linkNode ()) << " p state:" << m_player->process ()->state () << endl;
     if (ns == Node::state_deactivated && elm == m_document && !m_back_request) {
         emit endOfPlayItems (); // played all items
     } else if ((ns == Node::state_deactivated || ns == Node::state_finished) &&
-             m_current && m_current->isPlayable () &&
-             elm == m_current->mrl ()->realMrl ()) {
+             m_player->process ()->mrl() &&
+             elm == m_player->process ()->mrl ()->mrl ()->linkNode ()) {
         if (m_player->process ()->state () > Process::Ready)
             //a SMIL movies stopped by SMIL events rather than movie just ending
             m_player->process ()->stop ();
@@ -1108,31 +1114,16 @@ void Source::stateElementChanged (Node * elm, Node::State os, Node::State ns) {
     }
 }
 
-void Source::setEventDispatcher (NodePtr e) {
+SurfacePtr Source::getSurface (NodePtr n) {
     if (m_player->view ())
-        static_cast <View*> (m_player->view())->viewArea()->setEventListener(e);
+        return static_cast <View*>(m_player->view())->viewArea()->getSurface(n);
+    return 0L;
 }
 
 void Source::setInfoMessage (const QString & msg) {
     m_player->updateInfo (msg);
 }
 
-void Source::repaintRect (int x, int y, int w, int h) {
-    //kdDebug () << "repaint " << x << "," << y << " " << w << "x" << h << endl;
-    if (m_player->view ())
-     static_cast<View*>(m_player->view())->viewArea()->scheduleRepaint(x,y,w,h);
-}
-
-void Source::moveRect (int x, int y, int w, int h, int x1, int y1) {
-    if (m_player->view ())
-      static_cast<View*>(m_player->view())->viewArea()->moveRect(x,y,w,h,x1,y1);
-}
-
-void Source::avWidgetSizes (int x, int y, int w, int h, unsigned int * bg) {
-    if (m_player->view ())
-      static_cast<View*>(m_player->view())->viewArea ()->setAudioVideoGeometry (x, y, w, h, bg);
-}
-
 void Source::bitRates (int & preferred, int & maximal) {
     preferred = 1024 * m_player->settings ()->prefbitrate;
     maximal= 1024 * m_player->settings ()->maxbitrate;
@@ -1364,7 +1355,11 @@ void Source::stateChange(Process *p, Process::State olds, Process::State news) {
         p->viewer()->view()->controlPanel()->setPlaying(news > Process::Ready);
         kdDebug () << "processState " << statemap[olds] << " -> " << statemap[news] << endl;
         m_player->updateStatus (i18n ("Player %1 %2").arg (p->name ()).arg (statemap[news]));
-        if (news == Process::Playing) {
+        if (!p->mrl () && news > Process::Ready) {
+            p->stop (); // reschedule for Ready state
+        } else if (news == Process::Playing) {
+            if (p->mrl ()->state == Element::state_deferred)
+                p->mrl ()->undefer ();
             p->viewer ()->view ()->playingStart ();
             emit startPlaying ();
         } else if (news == Process::NotRunning) {
@@ -1376,18 +1371,16 @@ void Source::stateChange(Process *p, Process::State olds, Process::State news) {
             if (olds > Process::Ready) {
                 NodePtr node = p->mrl (); // p->mrl is weak, needs check
                 Mrl * mrl = node ? node->mrl () : 0L;
-                if (mrl && mrl->active ()) { // if cause is eof
-                    if (!m_back_request && mrl->state ==Element::state_deferred)
-                        mrl->undefer ();
+                if (m_back_request && m_back_request->isPlayable ()) {
+                    if (m_back_request->mrl ()->view_mode == Mrl::SingleMode)
+                        // jump in pl
+                        m_current = m_back_request;
                     else
-                        mrl->finish (); // set node to finished
-                }
-                if (m_back_request && m_back_request->isPlayable ()) { // jump in pl
-                    m_current = m_back_request;
-                    if (m_current->id > SMIL::id_node_first &&
+                        // overlapping SMIL audio/video
+                        mrl->endOfFile ();
+                    if (m_current->id >= SMIL::id_node_first &&
                             m_current->id < SMIL::id_node_last) {
-                        // don't mess with SMIL, just play the damn link
-                        playCurrent ();
+                        playCurrent ();  // just play back_request
                     } else {
                         // sanitize pl having all parents of current activated
                         m_document->reset (); // deactivate everything
@@ -1396,11 +1389,16 @@ void Source::stateChange(Process *p, Process::State olds, Process::State news) {
                         m_current->activate (); // calls requestPlayUrl
                     }
                     m_back_request = 0L;
-                }
-                if (m_player->view() && (!mrl || mrl->view_mode != Mrl::Window))
+                } else
+                    mrl->endOfFile (); // set node to finished
+                if (m_player->view() &&
+                        (!mrl || mrl->view_mode != Mrl::WindowMode))
                     static_cast<View*>(m_player->view())->viewArea()->repaint();
             } else
                 QTimer::singleShot (0, this, SLOT (playCurrent ()));
+        } else if (news == Process::Buffering) {
+            if (p->mrl ()->mrl ()->view_mode != Mrl::SingleMode)
+                p->mrl ()->defer (); // paused the SMIL
         }
     }
 }
@@ -1455,7 +1453,7 @@ KDE_NO_EXPORT void URLSource::stopResolving () {
         for (SharedPtr <ResolveInfo> ri = m_resolve_info; ri; ri = ri->next)
             ri->job->kill ();
         m_resolve_info = 0L;
-        m_player->updateStatus (i18n ("Disonnected"));
+        m_player->updateStatus (i18n ("Disconnected"));
         m_player->setLoaded (100);
     }
 }
@@ -1483,7 +1481,7 @@ void URLSource::jump (NodePtr e) {
 void URLSource::deactivate () {
     activated = false;
     reset ();
-    setEventDispatcher (NodePtr ());
+    getSurface (0L);
 }
 
 QString URLSource::prettyName () {
@@ -1551,7 +1549,7 @@ KDE_NO_EXPORT void URLSource::read (NodePtr root, QTextStream & textstream) {
     if (!line.isNull ()) {
         NodePtr cur_elm = root;
         if (cur_elm->isPlayable ())
-            cur_elm = cur_elm->mrl ()->realMrl ();
+            cur_elm = cur_elm->mrl ()->linkNode ();
         if (cur_elm->mrl ()->mimetype == QString ("audio/x-scpls")) {
             bool groupfound = false;
             int nr = -1;
@@ -1609,6 +1607,8 @@ KDE_NO_EXPORT void URLSource::read (NodePtr root, QTextStream & textstream) {
             }
         } else if (line.lower () != QString ("[reference]")) do {
             QString mrl = line.stripWhiteSpace ();
+            if (line == QString ("--stop--"))
+                break;
             if (mrl.lower ().startsWith (QString ("asf ")))
                 mrl = mrl.mid (4).stripWhiteSpace ();
             if (!mrl.isEmpty () && !mrl.startsWith (QChar ('#')))
@@ -1703,9 +1703,9 @@ void URLSource::play () {
 }
 
 bool URLSource::requestPlayURL (NodePtr mrl) {
-    if (m_document != mrl->mrl ()->realMrl ()) {
+    if (m_document.ptr () != mrl->mrl ()->linkNode ()) {
         KURL base = m_document->mrl ()->src;
-        KURL dest = mrl->mrl ()->realMrl ()->mrl ()->absolutePath ();
+        KURL dest = mrl->mrl ()->linkNode ()->absolutePath ();
         // check if some remote playlist tries to open something local, but
         // do ignore unknown protocols because there are so many and we only
         // want to cache local ones.
@@ -1825,6 +1825,10 @@ bool DataCache::preserve (const QString & url) {
     return false;
 }
 
+bool DataCache::isPreserved (const QString & url) {
+    return preserve_map.find (url) != preserve_map.end ();
+}
+
 bool DataCache::unpreserve (const QString & url) {
     const PreserveMap::iterator it = preserve_map.find (url);
     if (it == preserve_map.end ())
@@ -1835,7 +1839,7 @@ bool DataCache::unpreserve (const QString & url) {
 }
 
 RemoteObjectPrivate::RemoteObjectPrivate (RemoteObject * r)
- : job (0L), remote_object (r) {
+ : job (0L), remote_object (r), preserve_wait (false) {
     if (!memory_cache)
         dataCacheDeleter.setObject (memory_cache, new DataCache);
 }
@@ -1874,6 +1878,7 @@ KDE_NO_EXPORT bool RemoteObjectPrivate::download (const QString & str) {
         //kdDebug () << "download preserved " << str << endl;
         connect (memory_cache, SIGNAL (preserveRemoved (const QString &)),
                  this, SLOT (cachePreserveRemoved (const QString &)));
+        preserve_wait = true;
     }
     return false;
 }
@@ -1883,6 +1888,10 @@ KDE_NO_EXPORT void RemoteObjectPrivate::clear () {
         job->kill (); // quiet, no result signal
         job = 0L;
         memory_cache->unpreserve (url);
+    } else if (preserve_wait) {
+        disconnect (memory_cache, SIGNAL (preserveRemoved (const QString &)),
+                    this, SLOT (cachePreserveRemoved (const QString &)));
+        preserve_wait = false;
     }
 }
 
@@ -1897,7 +1906,8 @@ KDE_NO_EXPORT void RemoteObjectPrivate::slotResult (KIO::Job * kjob) {
 
 KDE_NO_EXPORT
 void RemoteObjectPrivate::cachePreserveRemoved (const QString & str) {
-    if (str == url) {
+    if (str == url && !memory_cache->isPreserved (str)) {
+        preserve_wait = false;
         disconnect (memory_cache, SIGNAL (preserveRemoved (const QString &)),
                     this, SLOT (cachePreserveRemoved (const QString &)));
         download (str);
