@@ -1,6 +1,6 @@
-/* This file is part of the KMPlayer project
+/* This file is part of the KDE project
  *
- * Copyright (C) 2004-2006 Koos Vriezen <koos.vriezen@gmail.com>
+ * Copyright (C) 2004 Koos Vriezen <koos.vriezen@xs4all.nl>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,18 +26,23 @@
 #include <config.h>
 #include <sys/time.h>
 
+#include <qstring.h>
+
+#include "kmplayer_def.h"
 #include "kmplayertypes.h"
 #include "kmplayershared.h"
 
+class QTextStream;
+
 namespace KMPlayer {
 
 class Document;
 class Node;
-class TextNode;
 class Mrl;
+class Surface;
 class ElementPrivate;
+class RemoteObjectPrivate;
 class Visitor;
-class MediaInfo;
 
 /*
  * Base class for objects that will be used as SharedPtr/WeakPtr pointers.
@@ -52,6 +57,8 @@ public:
     typedef SharedPtr <T> SharedType;
     typedef WeakPtr <T> WeakType;
 
+    virtual ~Item () {}
+
     SharedType self () const { return m_self; }
 protected:
     Item ();
@@ -64,7 +71,7 @@ private:
  * Because of the m_self member of Item<T>, it's not allowed to assign a
  * Item<T>* directly to SharedPtr<Item<T>>. Item<T>* will then reside in
  * two independent SharedData<Item<T>> objects.
- * So specialize constructor and assignment operators to fetch the
+ * So specialize constructor and assignment operators to fetch the 
  * SharedData<Item<T>> from the Item<T>* instead of creating a new one
  */
 #define ITEM_AS_POINTER(CLASS)                                         \
@@ -109,22 +116,19 @@ template <class T>
 class KMPLAYER_EXPORT List : public Item <List <T> > {
 public:
     List () {}
-    List (typename Item<T>::SharedType f, typename Item<T>::SharedType l)
+    List (typename Item<T>::SharedType f, typename Item<T>::SharedType l) 
         : m_first (f), m_last (l) {}
     ~List () { clear (); }
 
-    T* first () const { return m_first.ptr (); }
-    T* last () const { return m_last.ptr (); }
-    void append (T *c);
-    void insertBefore (T *c, T *b);
-    void remove (T *c);
+    typename Item<T>::SharedType first () const { return m_first; }
+    typename Item<T>::SharedType last () const { return m_last; }
+    void append (typename Item<T>::SharedType c);
+    void insertBefore(typename Item<T>::SharedType c, typename Item<T>::SharedType b);
+    void remove (typename Item<T>::SharedType c);
     void clear ();
     unsigned int length () const;
-    T* item (int i) const;
-
+    typename Item<T>::SharedType item (int i) const;
 protected:
-    void removeImpl (T *c);
-
     typename Item<T>::SharedType m_first;
     typename Item<T>::WeakType m_last;
 };
@@ -137,8 +141,10 @@ template <class T>
 class KMPLAYER_EXPORT ListNodeBase : public Item <T> {
     friend class List<T>;
 public:
-    T* nextSibling () const { return m_next.ptr (); }
-    T* previousSibling () const { return m_prev.ptr (); }
+    virtual ~ListNodeBase () {}
+
+    typename Item<T>::SharedType nextSibling () const { return m_next; }
+    typename Item<T>::SharedType previousSibling () const { return m_prev; }
 protected:
     ListNodeBase () {}
     typename Item<T>::SharedType m_next;
@@ -162,20 +168,18 @@ public:
 template <class T>
 class KMPLAYER_EXPORT TreeNode : public ListNodeBase <T> {
 public:
-    void insertBefore (T *c, T *b);
-    void appendChild (T *c);
-    void removeChild (typename Item<T>::SharedType c);
+    virtual ~TreeNode () {}
+
+    virtual void appendChild (typename Item<T>::SharedType c);
+    virtual void removeChild (typename Item<T>::SharedType c);
 
     bool hasChildNodes () const { return m_first_child != 0L; }
-    T* parentNode () const { return m_parent.ptr (); }
-    T* firstChild () const { return m_first_child.ptr (); }
-    T* lastChild () const { return m_last_child.ptr (); }
+    typename Item<T>::SharedType parentNode () const { return m_parent; }
+    typename Item<T>::SharedType firstChild () const { return m_first_child; }
+    typename Item<T>::SharedType lastChild () const { return m_last_child; }
 
 protected:
     TreeNode () {}
-    void insertBeforeImpl (T *c, T *b);
-    void appendChildImpl (T *c);
-    void removeChildImpl (typename Item<T>::SharedType c);
     typename Item<T>::WeakType m_parent;
     typename Item<T>::SharedType m_first_child;
     typename Item<T>::WeakType m_last_child;
@@ -187,15 +191,15 @@ protected:
 class KMPLAYER_EXPORT Attribute : public ListNodeBase <Attribute> {
 public:
     KDE_NO_CDTOR_EXPORT Attribute () {}
-    Attribute (const TrieString & n, const String & v);
+    Attribute (const TrieString & n, const QString & v);
     KDE_NO_CDTOR_EXPORT ~Attribute () {}
     TrieString name () const { return m_name; }
-    String value () const { return m_value; }
+    QString value () const { return m_value; }
     void setName (const TrieString &);
-    void setValue (const String &);
+    void setValue (const QString &);
 protected:
     TrieString m_name;
-    String m_value;
+    QString m_value;
 };
 
 ITEM_AS_POINTER(KMPlayer::Attribute)
@@ -204,7 +208,6 @@ ITEM_AS_POINTER(KMPlayer::Attribute)
  * Object should scale according the passed Fit value in SizedEvent
  */
 enum Fit {
-    fit_default,   // fill hidden
     fit_fill,      // fill complete area, no aspect preservation
     fit_hidden,    // keep aspect and don't scale, cut off what doesn't fit
     fit_meet,      // keep aspect and scale so that the smallest size just fits
@@ -212,56 +215,45 @@ enum Fit {
     fit_scroll     // keep aspect and don't scale, add scollbars if needed
 };
 
-enum MessageType
-{
-    MsgEventTimer = 0,
-    MsgEventClicked,
-    MsgEventPointerMoved,
-    MsgEventPointerInBounds,
-    MsgEventPointerOutBounds,
-    MsgEventStarting,
-    MsgEventStarted,
-    MsgEventStopped,
-    MsgMediaFinished,
-    // the above messages are ordered like SMIL timing events
-    MsgMediaReady,
-    MsgMediaUpdated,
-    MsgEventPostponed,
-    MsgSurfaceBoundsUpdate,      // bool parent surface resized
-    MsgSurfaceUpdate,
-    MsgSurfaceAttach,
-    MsgStateFreeze,
-    MsgStateRewind,
-    MsgChildReady,
-    MsgChildFinished,
-
-    MsgStartQueryMessage,
-    MsgQueryReady,
-    MsgQueryActorAgent,
-    MsgQueryRoleTiming,
-    MsgQueryRoleDisplay,
-    MsgQueryRoleChildDisplay,    // Mrl*
-    MsgQueryRoleSizer,
-    MsgQueryReceivers,
-    MsgEndQueryMessage,
-
-    MsgInfoString                // String*
+/*
+ * A generic event type
+ */
+class KMPLAYER_EXPORT Event : public Item <Event> {
+public:
+    KDE_NO_CDTOR_EXPORT Event (unsigned int event_id) : m_event_id (event_id) {}
+    KDE_NO_CDTOR_EXPORT virtual ~Event () {}
+    KDE_NO_EXPORT unsigned int id () const { return m_event_id; }
+protected:
+    unsigned int m_event_id;
 };
 
-#define MsgUnhandled ((void *) 357L)
+ITEM_AS_POINTER(KMPlayer::Event)
 
-#define MsgBool(x)                                                          \
-    (void*)(long)(x)
+/**
+ * Event signaling that attached region is resized
+ */
+class SizeEvent : public Event {
+public:
+    SizeEvent (Single x, Single y, Single w, Single h, Fit f);
+    Single x, y, w, h;
+    Fit fit;
+};
 
-#define nodeMessageReceivers(node, msg)                                     \
-    (NodeRefList*)(node)->message (MsgQueryReceivers, (void*)(long)(msg))
+extern const unsigned int event_pointer_clicked;
+extern const unsigned int event_pointer_moved;
+extern const unsigned int event_inbounds;
+extern const unsigned int event_outbounds;
+extern const unsigned int event_sized;
+extern const unsigned int event_postponed;
+extern const unsigned int event_timer;
+extern const unsigned int mediatype_attached;
 
 // convenient types
-typedef void Role;
 typedef Item<Node>::SharedType NodePtr;
 typedef Item<Node>::WeakType NodePtrW;
 typedef Item<Attribute>::SharedType AttributePtr;
 typedef Item<Attribute>::WeakType AttributePtrW;
+typedef Item<Event>::SharedType EventPtr;
 typedef List<Node> NodeList;                 // eg. for Node's children
 typedef Item<NodeList>::SharedType NodeListPtr;
 typedef Item<NodeList>::WeakType NodeListPtrW;
@@ -271,23 +263,18 @@ typedef Item<AttributeList>::SharedType AttributeListPtr;
 ITEM_AS_POINTER(KMPlayer::AttributeList)
 typedef ListNode<NodePtrW> NodeRefItem;      // Node for ref Nodes
 ITEM_AS_POINTER(KMPlayer::NodeRefItem)
-typedef ListNode<NodePtr> NodeStoreItem;   // list stores Nodes
-ITEM_AS_POINTER(KMPlayer::NodeStoreItem)
+//typedef ListNode<NodePtr> NodeStoreItem;   // list stores Nodes
 typedef NodeRefItem::SharedType NodeRefItemPtr;
 typedef NodeRefItem::WeakType NodeRefItemPtrW;
-typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event receivers
-typedef List<NodeStoreItem> NodeStoreList;
+typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event listeners
 typedef Item<NodeRefList>::SharedType NodeRefListPtr;
 typedef Item<NodeRefList>::WeakType NodeRefListPtrW;
 ITEM_AS_POINTER(KMPlayer::NodeRefList)
-
-template <> void List<NodeRefItem>::remove (NodeRefItem *c);
-template <> void TreeNode<Node>::appendChild (Node *c);
-template <> void TreeNode<Node>::insertBefore (Node *c, Node *b);
-template <> void TreeNode<Node>::removeChild (NodePtr c);
+typedef Item<Surface>::SharedType SurfacePtr;
+typedef Item<Surface>::WeakType SurfacePtrW;
 
 /*
- * Weak ref of the receivers list from signaler and the listener node
+ * Weak ref of the listeners list from signaler and the listener node
  */
 class KMPLAYER_EXPORT Connection {
     friend class Node;
@@ -296,25 +283,18 @@ public:
     void disconnect ();
     NodePtrW connectee; // the one that will, when ever, trigger the event
 private:
-    Connection (NodeRefList *ls, Node *node, Node *invoker);
+    Connection (NodeRefListPtr ls, NodePtr node, NodePtr invoker);
     NodeRefListPtrW listeners;
     NodeRefItemPtrW listen_item;
 };
 
 typedef SharedPtr <Connection> ConnectionPtr;
 
-struct XMLStringlet {
-    const String str;
-    XMLStringlet (const String & s) : str (s) {}
-};
-
-TextStream & operator << (TextStream &out, const XMLStringlet &txt);
-
 /*
  * Base class for XML nodes. Provides a w3c's DOM like API
  *
  * Most severe traps with using SharedPtr/WeakPtr for tree nodes:
- * - pointer ends up in two independent shared objects (hopefully with 
+ * - pointer ends up in two independent shared objects (hopefully with
  *   template specialization for constructor for T* and assignment of T* should
  *   be enough of defences ..)
  * - Node added two times (added ASSERT in appendChild/insertBefore)
@@ -331,8 +311,7 @@ class KMPLAYER_EXPORT Node : public TreeNode <Node> {
 public:
     enum State {
         state_init, state_deferred,
-        state_activated, state_began, state_finished,
-        state_deactivated, state_resetting
+        state_activated, state_began, state_finished, state_deactivated
     };
     enum PlayType {
         play_type_none, play_type_unknown, play_type_info,
@@ -341,27 +320,25 @@ public:
     virtual ~Node ();
     Document * document ();
     virtual Mrl * mrl ();
-    virtual Node *childFromTag (const String & tag);
-    void characterData (const String & s);
-    String innerText () const;
-    String innerXML () const;
-    String outerXML (bool inter_doc=true) const;
+    virtual NodePtr childFromTag (const QString & tag);
+    void characterData (const QString & s);
+    QString innerText () const;
+    QString innerXML () const;
+    QString outerXML () const;
     virtual const char * nodeName () const;
-    virtual String nodeValue () const;
+    virtual QString nodeValue () const;
+    virtual void setNodeName (const QString &) {}
 
     /**
      * If this is a derived Mrl object and has a SRC attribute
      */
     virtual PlayType playType ();
     bool isPlayable () { return playType () > play_type_none; }
-    virtual bool isElementNode () const { return false; }
+    virtual bool isElementNode () { return false; }
     /**
      * If this node should be visible to the user
      */
     virtual bool expose () const;
-    virtual String caption () const;
-    virtual void setCaption (const String &t);
-    virtual String description () const;
     /**
      * If this node should be visible to the user
      */
@@ -377,19 +354,28 @@ public:
      * Return a NULL ptr if event_id is not supported.
      * \sa: Connection::disconnect()
      */
-    ConnectionPtr connectTo (Node *node, MessageType msg);
+    ConnectionPtr connectTo (NodePtr node, unsigned int event_id);
     /*
      * Event send to this node, return true if handled
      */
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual bool handleEvent (EventPtr event);
     /*
-     * Dispatch Event to all connectorss of MessageType
+     * Dispatch Event to all listeners of event->id()
      */
-    void deliver (MessageType msg, void *content);
+    void propagateEvent (EventPtr event);
     /**
      * Alternative to event handling is the Visitor pattern
      */
     virtual void accept (Visitor *);
+    /*
+     * Returns a listener list for event_id, or a null ptr if not supported.
+     */
+    virtual NodeRefListPtr listeners (unsigned int event_id);
+    /**
+     * Adds node to call 'handleEvent()' for all events that gets
+     * delivered to this node, ignored by default
+     */
+    virtual SurfacePtr getSurface (NodePtr node);
     /**
      * Activates element, sets state to state_activated. Will call activate() on
      * firstChild or call deactivate().
@@ -410,7 +396,7 @@ public:
      */
     virtual void defer ();
     /**
-     * Puts a deferred element in activated again, calls activate() again
+     * Puts a deferred element in activated again, calls activate() again 
      */
     virtual void undefer ();
     /**
@@ -419,11 +405,11 @@ public:
     virtual void begin ();
     /**
      * Sets state to state_finish when >= state_activated.
-     * Notifies parent with a MsgChildFinished message.
+     * Notifies parent with a childDone call.
      */
     virtual void finish ();
     /**
-     * Stops element, sets state to state_deactivated. Calls deactivate() on
+     * Stops element, sets state to state_deactivated. Calls deactivate() on 
      * activated/deferred children. May call childDone() when active() and not
      * finished yet.
      */
@@ -433,10 +419,21 @@ public:
      * state to state_init.
      */
     virtual void reset ();
+    /**
+     * Notification from child that it has began.
+     */
+    virtual void childBegan (NodePtr child);
+    /**
+     * Notification from child that it's finished. Will call deactivate() on
+     * child if it state is state_finished. Call activate() on nexSibling
+     * or deactivate() if there is none.
+     */
+    virtual void childDone (NodePtr child);
     virtual void clear ();
     void clearChildren ();
-    using TreeNode<Node>::insertBefore;
-    void insertBefore (const String &xml, Node *b);
+    void appendChild (NodePtr c);
+    void insertBefore (NodePtr c, NodePtr b);
+    void removeChild (NodePtr c);
     void replaceChild (NodePtr _new, NodePtr old);
     /*
      * Get rid of whitespace only text nodes
@@ -460,24 +457,21 @@ protected:
 public:
     State state;
     short id;
-    bool auxiliary_node : 1;
-    bool editable : 1;
-    bool open : 1;
+private:
+    bool auxiliary_node;
+protected:
+    bool editable;
 };
 
 ITEM_AS_POINTER(KMPlayer::Node)
 
 const short id_node_document = 1;
-const short id_node_record_document = 2;
 const short id_node_text = 5;
 const short id_node_cdata = 6;
 
 const short id_node_group_node = 25;
 const short id_node_playlist_document = 26;
 const short id_node_playlist_item = 27;
-const short id_node_param = 28;
-const short id_node_svg = 29;
-
 /*
  * Element node, XML node that can have attributes
  */
@@ -485,26 +479,25 @@ class KMPLAYER_EXPORT Element : public Node {
 public:
     ~Element ();
     void setAttributes (AttributeListPtr attrs);
-    void setAttribute (const TrieString & name, const String & value);
-    String getAttribute (const TrieString & name);
-    KDE_NO_EXPORT AttributeList *attributes () const { return m_attributes.ptr (); }
+    void setAttribute (const TrieString & name, const QString & value);
+    QString getAttribute (const TrieString & name);
+    KDE_NO_EXPORT AttributeListPtr attributes () const { return m_attributes; }
     virtual void init ();
-    virtual void reset ();
+    virtual void deactivate ();
     virtual void clear ();
-    virtual bool isElementNode () const { return true; }
-    virtual void accept (Visitor * v);
+    virtual bool isElementNode () { return true; }
     /**
      * Params are like attributes, but meant to be set dynamically. Caller may
      * pass a modification id, that it can use to restore the old value.
      * Param will be auto removed on deactivate
      */
-    void setParam (const TrieString &para, const String &val, int * mod_id=0L);
-    String param (const TrieString & para);
+    void setParam (const TrieString &para, const QString &val, int * mod_id=0L);
+    QString param (const TrieString & para);
     void resetParam (const TrieString & para, int mod_id);
     /**
      * Called from (re)setParam for specialized interpretation of params
      **/
-    virtual void parseParam (const TrieString &, const String &) {}
+    virtual void parseParam (const TrieString &, const QString &) {}
 protected:
     Element (NodePtr & d, short id=0);
     AttributeListPtr m_attributes;
@@ -512,79 +505,70 @@ private:
     ElementPrivate * d;
 };
 
+/**
+ * Node that references another node
+ */
+class RefNode : public Node {
+public:
+    RefNode (NodePtr & d, NodePtr ref);
+    virtual const char * nodeName () const { return tag_name.ascii (); }
+    NodePtr refNode () const { return ref_node; }
+    void setRefNode (const NodePtr ref);
+protected:
+    NodePtrW ref_node;
+    QString tag_name;
+};
+
 template <class T>
 inline KDE_NO_EXPORT T * convertNode (NodePtr e) {
     return static_cast <T *> (e.ptr ());
 }
 
-KMPLAYER_NO_EXPORT
-inline Node *findChildWithId (const Node *p, const short id) {
-    for (Node *c = p->firstChild (); c; c = c->nextSibling ())
-        if (id == c->id)
-            return c;
-    return NULL;
-}
-
-class KMPLAYER_EXPORT Title : public Element {
-protected:
-    Title (NodePtr & d, short id=0) : Element (d, id ) {}
-
-public:
-    virtual String caption () const;
-    virtual void setCaption (const String &t);
-
-    String title;
-};
-
 /**
  * Element representing a playable link, like URL to a movie or playlist.
  */
-class KMPLAYER_EXPORT Mrl : public Title {
+class KMPLAYER_EXPORT Mrl : public Element {
 protected:
     Mrl (NodePtr & d, short id=0);
-    Node *childFromTag (const String & tag);
-    void parseParam (const TrieString &, const String &);
+    NodePtr childFromTag (const QString & tag);
+    void parseParam (const TrieString &, const QString &);
     unsigned int cached_ismrl_version;
     PlayType cached_play_type;
-
 public:
-    enum { SingleMode = 0, WindowMode };
-
     ~Mrl ();
     PlayType playType ();
     /*
      * The original node (or this) having the URL, needed for playlist expansion
-     */
+     */ 
     virtual Mrl * linkNode ();
     virtual Mrl * mrl ();
-    String absolutePath ();
-
+    virtual void endOfFile ();
+    QString absolutePath ();
+    /*
+     * Reimplement to callback with requestPlayURL if isPlayable()
+     */ 
     virtual void activate ();
-    virtual void begin ();
-    virtual void defer ();
-    virtual void undefer ();
-    virtual void deactivate ();
-    virtual void *message (MessageType msg, void *content=NULL);
-
-    static unsigned int parseTimeString (const String &s);
-    static String *parsePanZoomString (const String &pz, String *coord);
+    /**
+     * By default support one event handler (eg. SMIL or RP child document)
+     */
+    virtual SurfacePtr getSurface (NodePtr node);
+    virtual bool handleEvent (EventPtr event);
 
     /**
      * If this Mrl is top node of external document, opener has the
      * location in SCR. Typically that's the parent of this node.
      */
     NodePtrW opener; //if this node is top node of external document,
-    MediaInfo *media_info;
-    String src;
-    String mimetype;
-    SSize size;
+    QString src;
+    QString pretty_name;
+    QString mimetype;
+    int width;
+    int height;
     float aspect;
     int repeat;
-    unsigned int clip_begin;
-    unsigned char view_mode;
+    enum { SingleMode = 0, WindowMode } view_mode;
     bool resolved;
     bool bookmarkable;
-    bool audio_only;
 };
 
 /**
@@ -592,10 +576,28 @@ public:
  */
 class KMPLAYER_EXPORT PlayListNotify {
 public:
+    virtual ~PlayListNotify () {}
+    /**
+     * Ask for playing a video/audio mrl by backend players
+     * If returning false, the element will be set to finished
+     */
+    virtual bool requestPlayURL (NodePtr mrl) = 0;
+    /**
+     * Called by an unresolved Mrl, check if this node points to a playlist
+     */
+    virtual bool resolveURL (NodePtr mrl) = 0;
     /**
      * Element has activated or deactivated notification
      */
-    virtual void stateElementChanged(Node *n, Node::State os, Node::State ns)=0;
+    virtual void stateElementChanged (Node * element, Node::State old_state, Node::State new_state) = 0;
+    /**
+     * Set element to which to send GUI events and return a surface for drawing
+     */
+    virtual SurfacePtr getSurface (NodePtr node) = 0;
+    /**
+     * Request to show msg for informing the user
+     */
+    virtual void setInfoMessage (const QString & msg) = 0;
     /**
      * Ask for connection bitrates settings
      */
@@ -604,70 +606,82 @@ public:
      * Sets next call to Document::timer() or -1 to cancel a previous call
      */
     virtual void setTimeout (int ms) = 0;
-    /**
-     * Request to open url with mimetype
-     */
-    virtual void openUrl (const URL &, const String &tgt, const String &srv)=0;
-    /**
-     * Change repaint updaters
-     */
-    virtual void addRepaintUpdater (Node *node)=0;
-    virtual void removeRepaintUpdater (Node *node)=0;
-    virtual void enableRepaintUpdaters (bool enable, unsigned int off_time)=0;
-};
-
-struct KMPLAYER_EXPORT Command {
-    virtual void apply (struct timeval &timeout) = 0;
-    virtual bool postponable () const = 0;
-    virtual int delay () const = 0;
-    virtual void removed () = 0;
-};
-
-/*
- *  A generic type for posting messages
- **/
-class KMPLAYER_EXPORT Posting : public Command {
-public:
-    KDE_NO_CDTOR_EXPORT Posting (Node *n, Node *tgt, MessageType msg)
-        : source (n), target (tgt), message (msg) {}
-    KDE_NO_CDTOR_EXPORT virtual ~Posting () {}
-
-    virtual void apply (struct timeval &timeout);
-    virtual bool postponable () const;
-    virtual int delay () const;
-    virtual void removed ();
-
-    NodePtrW source;
-    NodePtrW target;
-    MessageType message;
 };
 
 /**
- * Posting signaling a timer event
+ * Base class for cached network data
  */
-class KMPLAYER_NO_EXPORT TimerPosting : public Posting {
+class KMPLAYER_NO_EXPORT RemoteObject {
+    friend class RemoteObjectPrivate;
 public:
-    TimerPosting (Node *t, int ms, unsigned eid=0);
-
-    virtual void apply (struct timeval &timeout);
-    virtual int delay () const;
-
-    unsigned event_id;
-    int milli_sec;
-    bool interval; // set to 'true' in 'Node::message()' to make it repeat
+    RemoteObject ();
+    virtual ~RemoteObject ();
+    bool wget (const QString & url);
+    void killWGet ();
+    void clear ();
+    QString mimetype ();
+protected:
+    KDE_NO_EXPORT virtual void remoteReady (QByteArray &) {}
+    bool downloading () const;
+private:
+    RemoteObjectPrivate *d;
 };
 
-class KMPLAYER_NO_EXPORT UpdateEvent {
+class KMPLAYER_NO_EXPORT Surface : public TreeNode <Surface> {
 public:
-    UpdateEvent (Document *, unsigned int off_time);
-    unsigned int cur_event_time;
-    unsigned int skipped_time;
+    Surface (NodePtr node, const SRect & rect);
+    ~Surface();
+
+    virtual SurfacePtr createSurface (NodePtr owner, const SRect & rect) = 0;
+    virtual void toScreen (Single & x, Single & y, Single & w, Single & h) = 0;
+    virtual void resize (const SRect & rect) = 0;
+    virtual void repaint (Single x, Single y, Single w, Single h) = 0;
+    virtual void video (Single x, Single y, Single w, Single h) = 0;
+
+    NodePtrW node;
+    SRect bounds;                  // bounds in in parent coord. 
+    Single xoffset, yoffset;       // translation for aspects
+    float xscale, yscale;          // internal scaling
+    unsigned int background_color; // rgba background color
+protected:
+    Surface (const SRect & rect);
+};
+
+ITEM_AS_POINTER(KMPlayer::Surface)
+
+/**
+ * To have a somewhat synchronized time base, node having timers should use
+ * this. Idea is that if a node still waiting for network data, it can hold
+ * this time line.
+ */
+class KMPLAYER_NO_EXPORT TimerInfo : public ListNodeBase <TimerInfo> {
+public:
+    TimerInfo (NodePtr n, unsigned id, struct timeval & now, int ms);
+    KDE_NO_CDTOR_EXPORT ~TimerInfo () {}
+    NodePtrW node;
+    unsigned event_id;
+    struct timeval timeout;
+    int milli_sec;
+};
+
+ITEM_AS_POINTER(KMPlayer::TimerInfo)
+typedef Item <TimerInfo>::SharedType TimerInfoPtr;
+typedef Item <TimerInfo>::WeakType TimerInfoPtrW;
+
+/**
+ * Event signaling a timer event
+ */
+class KMPLAYER_NO_EXPORT TimerEvent : public Event {
+public:
+    TimerEvent (TimerInfoPtr tinfo);
+    TimerInfoPtrW timer_info;
+    bool interval; // set to 'true' in 'Node::handleEvent()' to make it repeat
 };
 
 /**
  * Event signaling postponed or proceeded
  */
-class KMPLAYER_NO_EXPORT PostponedEvent {
+class KMPLAYER_NO_EXPORT PostponedEvent : public Event {
 public:
     PostponedEvent (bool postponed);
     bool is_postponed; // postponed or proceeded
@@ -689,93 +703,113 @@ public:
 typedef SharedPtr <Postpone> PostponePtr;
 typedef WeakPtr <Postpone> PostponePtrW;
 
-struct KMPLAYER_NO_EXPORT CommandData {
-    CommandData (Command *e, CommandData *n);
-    ~CommandData ();
-
-    Command *event;
-    struct timeval timeout;
-
-    CommandData *next;
-};
-
 /**
  * The root of the DOM tree
  */
 class KMPLAYER_EXPORT Document : public Mrl {
     friend class Postpone;
 public:
-    Document (const String &, PlayListNotify * notify = 0L);
+    Document (const QString &, PlayListNotify * notify = 0L);
     ~Document ();
-    Node *getElementById (const String & id);
-    Node *getElementById (Node *start, const String & id, bool inter_doc);
+    NodePtr getElementById (const QString & id);
+    NodePtr getElementById (NodePtr start, const QString & id, bool inter_doc);
     /** All nodes have shared pointers to Document,
      * so explicitly dispose it (calls clear and set m_doc to 0L)
      * */
     void dispose ();
-    virtual Node *childFromTag (const String & tag);
+    virtual NodePtr childFromTag (const QString & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "document"; }
     virtual void activate ();
     virtual void defer ();
     virtual void undefer ();
     virtual void reset ();
-
-    Command *post (Command *event);
-    void cancelPosting (Command *event);
-    void pausePosting (Command *e);
-    void unpausePosting (Command *e, int ms);
-
+    /**
+     * Ask for TimerEvent for Node n in ms milli-seconds.
+     * Returns weak ref to TimerInfo ptr, which is an item in the timers list
+     */
+    TimerInfoPtrW setTimeout (NodePtr n, int ms, unsigned id=0);
+    void cancelTimer (TimerInfoPtr ti);
     void timeOfDay (struct timeval &);
     PostponePtr postpone ();
-    bool postponed () const { return !!postpone_ref || !! postpone_lock; }
     /**
-     * Called by PlayListNotify, processes events in event_queue with timeout set to now
+     * Called by PlayListNotify, creates TimerEvent on first item in timers. 
+     * Returns true if to repeat this same timeout FIXME.
      */
-    void timer ();
-    void updateTimeout ();
+    bool timer ();
     /**
-     * Document has list of postponed receivers, eg. for running (gif)movies
+     * Document has list of postponed listeners, eg. for running (gif)movies
      */
-    virtual void *message (MessageType msg, void *content=NULL);
+    virtual NodeRefListPtr listeners (unsigned int event_id);
+    /**
+     * Reimplement, so it will call PlayListNotify::getSurface()
+     */
+    virtual SurfacePtr getSurface (NodePtr node);
 
-    void insertPosting (Command *e, const struct timeval &tv);
-
-    PlayListNotify *notify_listener;
+    List <TimerInfo> timers; //FIXME: make as connections
+    PlayListNotify * notify_listener;
     unsigned int m_tree_version;
     unsigned int last_event_time;
 private:
     void proceed (const struct timeval & postponed_time);
-    void setNextTimeout (const struct timeval &now);
-
     PostponePtrW postpone_ref;
     PostponePtr postpone_lock;
     NodeRefListPtr m_PostponedListeners;
-    CommandData *event_queue;
-    CommandData *paused_queue;
-    CommandData *cur_event;
+    int cur_timeout;
     struct timeval first_event_time;
-    int cur_time_interval;
+    bool intimer;
+};
+
+/**
+ * Represents XML text, like "some text" in '<foo>some text</foo>'
+ */
+class KMPLAYER_EXPORT TextNode : public Node {
+public:
+    TextNode (NodePtr & d, const QString & s, short _id = id_node_text);
+    KDE_NO_CDTOR_EXPORT ~TextNode () {}
+    void appendText (const QString & s);
+    void setText (const QString & txt) { text = txt; }
+    const char * nodeName () const { return "#text"; }
+    QString nodeValue () const;
+    bool expose () const;
+protected:
+    QString text;
+};
+
+/**
+ * Represents cdata sections, like "some text" in '<![CDATA[some text]]>'
+ */
+class KMPLAYER_EXPORT CData : public TextNode {
+public:
+    CData (NodePtr & d, const QString & s);
+    KDE_NO_CDTOR_EXPORT ~CData () {}
+    const char * nodeName () const { return "#cdata"; }
+};
+
+/**
+ * Unrecognized tag by parent element or just some auxiliary node
+ */
+class DarkNode : public Element {
+public:
+    DarkNode (NodePtr & d, const QString & n, short id=0);
+    KDE_NO_CDTOR_EXPORT ~DarkNode () {}
+    const char * nodeName () const { return name.ascii (); }
+    NodePtr childFromTag (const QString & tag);
+    virtual bool expose () const;
+protected:
+    QString name;
 };
 
 namespace SMIL {
-    class Smil;
-    class Layout;
     class RegionBase;
-    class Seq;
-    class Switch;
-    class Par;
-    class Excl;
-    class Transition;
-    class Animate;
-    class AnimateMotion;
+    class Region;
+    class Layout;
     class MediaType;
     class ImageMediaType;
     class TextMediaType;
     class RefMediaType;
+    class AVMediaType;
     class Brush;
-    class SmilText;
-    class TextFlow;
-    class PriorityClass;
+    class TimedMrl;
     class Anchor;
     class Area;
 }
@@ -795,26 +829,15 @@ public:
     KDE_NO_CDTOR_EXPORT Visitor () {}
     KDE_NO_CDTOR_EXPORT virtual ~Visitor () {}
     virtual void visit (Node *) {}
-    virtual void visit (TextNode *);
-    virtual void visit (Element *);
-    virtual void visit (SMIL::Smil *) {}
+    virtual void visit (SMIL::Region *);
     virtual void visit (SMIL::Layout *);
-    virtual void visit (SMIL::RegionBase *);
-    virtual void visit (SMIL::Seq *);
-    virtual void visit (SMIL::Switch *);
-    virtual void visit (SMIL::Par *);
-    virtual void visit (SMIL::Excl *);
-    virtual void visit (SMIL::Transition *);
-    virtual void visit (SMIL::Animate *);
-    virtual void visit (SMIL::AnimateMotion *);
-    virtual void visit (SMIL::PriorityClass *);
+    virtual void visit (SMIL::TimedMrl *);
     virtual void visit (SMIL::MediaType *);
     virtual void visit (SMIL::ImageMediaType *);
     virtual void visit (SMIL::TextMediaType *);
     virtual void visit (SMIL::RefMediaType *);
+    virtual void visit (SMIL::AVMediaType *);
     virtual void visit (SMIL::Brush *);
-    virtual void visit (SMIL::SmilText *);
-    virtual void visit (SMIL::TextFlow *);
     virtual void visit (SMIL::Anchor *);
     virtual void visit (SMIL::Area *);
     virtual void visit (RP::Imfl *) {}
@@ -827,47 +850,6 @@ public:
     virtual void visit (RP::Animate *) {}
 };
 
-/**
- * Represents XML text, like "some text" in '<foo>some text</foo>'
- */
-class KMPLAYER_EXPORT TextNode : public Node {
-public:
-    TextNode (NodePtr & d, const String & s, short _id = id_node_text);
-    KDE_NO_CDTOR_EXPORT ~TextNode () {}
-    void appendText (const String & s);
-    void setText (const String & txt) { text = txt; }
-    void accept (Visitor *v) { v->visit (this); }
-    const char * nodeName () const { return "#text"; }
-    String nodeValue () const;
-    bool expose () const;
-protected:
-    String text;
-};
-
-/**
- * Represents cdata sections, like "some text" in '<![CDATA[some text]]>'
- */
-class KMPLAYER_EXPORT CData : public TextNode {
-public:
-    CData (NodePtr & d, const String & s);
-    KDE_NO_CDTOR_EXPORT ~CData () {}
-    const char * nodeName () const { return "#cdata"; }
-};
-
-/**
- * Unrecognized tag by parent element or just some auxiliary node
- */
-class DarkNode : public Element {
-public:
-    DarkNode (NodePtr & d, const String & n, short id=0);
-    KDE_NO_CDTOR_EXPORT ~DarkNode () {}
-    const char * nodeName () const { return (const char *) name; }
-    Node *childFromTag (const String & tag);
-    virtual bool expose () const;
-protected:
-    String name;
-};
-
 //-----------------------------------------------------------------------------
 
 /**
@@ -875,7 +857,7 @@ protected:
  */
 class KMPLAYER_EXPORT GenericURL : public Mrl { 
 public:
-    GenericURL(NodePtr &d, const String &s, const String &n=String());
+    GenericURL(NodePtr &d, const QString &s, const QString &n=QString::null);
     KDE_NO_EXPORT const char * nodeName () const { return "url"; }
     void closed ();
 };
@@ -886,74 +868,62 @@ public:
 class KMPLAYER_EXPORT GenericMrl : public Mrl { 
 public:
     KDE_NO_CDTOR_EXPORT GenericMrl (NodePtr & d) : Mrl (d), node_name ("mrl") {}
-    GenericMrl(NodePtr &d, const String &s,
-            const String & nm = String(), const String &tag = String ("mrl"));
-    KDE_NO_EXPORT const char * nodeName () const
-        { return (const char *) node_name; }
+    GenericMrl(NodePtr &d, const QString &s, const QString & name=QString::null, const QString &tag=QString ("mrl"));
+    KDE_NO_EXPORT const char * nodeName () const { return node_name.ascii (); }
     void closed ();
     bool expose () const;
-    /**
-     * Will return false if this document has child nodes
-     */
-    String node_name;
+    QString node_name;
 };
 
 KMPLAYER_EXPORT
-void readXML (NodePtr root, TextStream & in, const String & firstline, bool set_opener=true);
-KMPLAYER_EXPORT Node * fromXMLDocumentTag (NodePtr & d, const String & tag);
+void readXML (NodePtr root, QTextStream & in, const QString & firstline, bool set_opener=true);
+KMPLAYER_EXPORT Node * fromXMLDocumentTag (NodePtr & d, const QString & tag);
 
 template <class T>
 inline Item<T>::Item () : m_self (static_cast <T*> (this), true) {}
 
-template <class T> inline void List<T>::append (T *c) {
+template <class T> inline void List<T>::append(typename Item<T>::SharedType c) {
     if (!m_first) {
-        m_first = c->m_self;
-        m_last = c->m_self;
+        m_first = m_last = c;
     } else {
-        m_last->m_next = c->m_self;
+        m_last->m_next = c;
         c->m_prev = m_last;
-        m_last = c->m_self;
+        m_last = c;
     }
 }
 
-template <class T> inline void List<T>::insertBefore (T *c, T *b) {
+template <class T> inline void List<T>::insertBefore(typename Item<T>::SharedType c, typename Item<T>::SharedType b) {
     if (!b) {
         append (c);
     } else {
-        c->m_next = b->m_self;
         if (b->m_prev) {
-            b->m_prev->m_next = c->m_self;
+            b->m_prev->m_next = c;
             c->m_prev = b->m_prev;
         } else {
             c->m_prev = 0L;
-            m_first = c->m_self;
+            m_first = c;
         }
-        b->m_prev = c->m_self;
+        b->m_prev = c;
+        c->m_next = b;
     }
 }
 
-template <class T> inline void List<T>::removeImpl (T *c) {
-    typename Item<T>::SharedType s = c->m_self;
-    if (c->m_prev)
+template <class T> inline void List<T>::remove(typename Item<T>::SharedType c) {
+    if (c->m_prev) {
         c->m_prev->m_next = c->m_next;
-    else
+    } else
         m_first = c->m_next;
     if (c->m_next) {
         c->m_next->m_prev = c->m_prev;
         c->m_next = 0L;
-    } else {
+    } else
         m_last = c->m_prev;
-    }
     c->m_prev = 0L;
 }
 
-template <class T> inline void List<T>::remove (T *c) {
-    removeImpl (c);
-}
-
 template <class T> inline unsigned int List<T>::length () const {
     unsigned int count = 0;
-    for (T *t = m_first.ptr (); t; t = t->nextSibling ())
+    for (typename Item<T>::SharedType t = m_first; t; t = t->nextSibling ())
         count++;
     return count;
 }
@@ -963,18 +933,17 @@ template <class T> inline void List<T>::clear () {
 }
 
 template <class T>
-inline T* List<T>::item (int i) const {
+inline typename Item<T>::SharedType List<T>::item (int i) const {
     for (typename Item<T>::SharedType t = m_first; t; t = t->nextSibling(), --i)
         if (i == 0)
-            return t.ptr ();
+            return t;
     return typename Item<T>::SharedType ();
 }
 
 template <class T>
-inline void TreeNode<T>::appendChildImpl (T *c) {
+inline void TreeNode<T>::appendChild (typename Item<T>::SharedType c) {
     if (!m_first_child) {
-        m_first_child = c;
-        m_last_child = c;
+        m_first_child = m_last_child = c;
     } else {
         m_last_child->m_next = c;
         c->m_prev = m_last_child;
@@ -984,25 +953,7 @@ inline void TreeNode<T>::appendChildImpl (T *c) {
 }
 
 template <class T>
-inline void TreeNode<T>::insertBeforeImpl (T *c, T *b) {
-    if (!b) {
-        appendChild (c);
-    } else {
-        c->m_next = b->m_self;
-        if (b->m_prev) {
-            b->m_prev->m_next = c->m_self;
-            c->m_prev = b->m_prev;
-        } else {
-            c->m_prev = 0L;
-            m_first_child = c->m_self;
-        }
-        b->m_prev = c->m_self;
-        c->m_parent = Item<T>::m_self;
-    }
-}
-
-template <class T>
-inline void TreeNode<T>::removeChildImpl (typename Item<T>::SharedType c) {
+inline void TreeNode<T>::removeChild (typename Item<T>::SharedType c) {
     if (c->m_prev) {
         c->m_prev->m_next = c->m_next;
     } else
