@@ -75,6 +75,7 @@ public:
     KDE_NO_EXPORT const QString & pipeCmd () const { return m_pipecmd; }
     KDE_NO_EXPORT const QString & options () const { return m_options; }
     KDE_NO_EXPORT const QString & recordCmd () const { return m_recordcmd; }
+    KDE_NO_EXPORT const QString & tuner () const { return m_tuner; }
     KDE_NO_EXPORT NodePtr current () const { return m_current; }
     virtual NodePtr document ();
     virtual NodePtr root ();
@@ -96,6 +97,7 @@ public:
     // backend process state changed
     virtual void stateChange (Process *, Process::State os, Process::State ns);
     KDE_NO_EXPORT void setAutoPlay (bool b) { m_auto_play = b; }
+    KDE_NO_EXPORT bool autoPlay () const { return m_auto_play; }
     void setTitle (const QString & title);
     void setLoading (int percentage);
     bool setCurrent (NodePtr mrl);
@@ -135,11 +137,8 @@ protected:
     bool requestPlayURL (NodePtr mrl);
     bool resolveURL (NodePtr mrl);
     void stateElementChanged (Node * element, Node::State os, Node::State ns);
-    void setEventDispatcher (NodePtr element);
+    SurfacePtr getSurface (NodePtr node);
     void setInfoMessage (const QString & msg);
-    void repaintRect (int x, int y, int w, int h);
-    void moveRect (int x, int y, int w, int h, int x1, int y1);
-    void avWidgetSizes (int x, int y, int w, int h, unsigned int * bg_color);
     void bitRates (int & preferred, int & maximal);
     void setTimeout (int ms);
 
@@ -156,6 +155,7 @@ protected:
     QString m_audiodevice;
     QString m_videodevice;
     QString m_videonorm;
+    QString m_tuner;
     int m_frequency;
     int m_xvport;
     int m_xvencoding;
