@@ -37,15 +37,12 @@
 #include <qslider.h>
 #include <qlabel.h>
 #include <qdatastream.h>
-#include <qpixmap.h>
-#include <qpainter.h>
 #include <qwidgetstack.h>
 #include <qheader.h>
 #include <qcursor.h>
 #include <qclipboard.h>
 
 #include <kiconloader.h>
-#include <kstaticdeleter.h>
 #include <kstatusbar.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -58,11 +55,13 @@
 #include <kfinddialog.h>
 #include <dcopclient.h>
 #include <kglobalsettings.h>
+#include <kstaticdeleter.h>
 
 #include "kmplayerview.h"
 #include "kmplayercontrolpanel.h"
 #include "kmplayersource.h"
 #include "playlistview.h"
+#include "viewarea.h"
 
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
@@ -81,331 +80,12 @@ extern const char * playlist_xpm[];
 
 
 /* mouse invisible: define the time (in 1/1000 seconds) before mouse goes invisible */
-#define MOUSE_INVISIBLE_DELAY 2000
 
 
 using namespace KMPlayer;
 
 //-------------------------------------------------------------------------
 
-KDE_NO_CDTOR_EXPORT ViewArea::ViewArea (QWidget * parent, View * view)
- : QWidget (parent, "kde_kmplayer_viewarea", WResizeNoErase | WRepaintNoErase),
-   m_parent (parent),
-   m_view (view),
-   m_painter (0L),
-   m_paint_buffer (0L),
-   m_collection (new KActionCollection (this)),
-   m_mouse_invisible_timer (0),
-   m_repaint_timer (0),
-   m_fullscreen_scale (100),
-   scale_lbl_id (-1),
-   scale_slider_id (-1),
-   m_fullscreen (false),
-   m_minimal (false) {
-    setEraseColor (QColor (0, 0, 0));
-    setAcceptDrops (true);
-    new KAction (i18n ("Fullscreen"), KShortcut (Qt::Key_F), this, SLOT (accelActivated ()), m_collection, "view_fullscreen_toggle");
-    setMouseTracking (true);
-}
-
-KDE_NO_CDTOR_EXPORT ViewArea::~ViewArea () {
-    delete m_painter;
-    delete m_paint_buffer;
-}
-
-KDE_NO_EXPORT void ViewArea::fullScreen () {
-    killTimers ();
-    m_mouse_invisible_timer = m_repaint_timer = 0;
-    if (m_fullscreen) {
-        showNormal ();
-        reparent (m_parent, 0, QPoint (0, 0), true);
-        static_cast <KDockWidget *> (m_parent)->setWidget (this);
-        for (unsigned i = 0; i < m_collection->count (); ++i)
-            m_collection->action (i)->setEnabled (false);
-        if (scale_lbl_id != -1) {
-            m_view->controlPanel ()->popupMenu ()->removeItem (scale_lbl_id);
-            m_view->controlPanel ()->popupMenu ()->removeItem (scale_slider_id);
-            scale_lbl_id = scale_slider_id = -1;
-        }
-        m_view->controlPanel ()->button (ControlPanel::button_playlist)->setIconSet (QIconSet (QPixmap (playlist_xpm)));
-    } else {
-        m_topwindow_rect = topLevelWidget ()->geometry ();
-        reparent (0L, 0, qApp->desktop()->screenGeometry(this).topLeft(), true);
-        showFullScreen ();
-        for (unsigned i = 0; i < m_collection->count (); ++i)
-            m_collection->action (i)->setEnabled (true);
-        QPopupMenu * menu = m_view->controlPanel ()->popupMenu ();
-        QLabel * lbl = new QLabel (i18n ("Scale:"), menu);
-        scale_lbl_id = menu->insertItem (lbl, -1, 4);
-        QSlider * slider = new QSlider (50, 150, 10, m_fullscreen_scale, Qt::Horizontal, menu);
-        connect (slider, SIGNAL (valueChanged (int)), this, SLOT (scale (int)));
-        scale_slider_id = menu->insertItem (slider, -1, 5);
-        m_view->controlPanel ()->button (ControlPanel::button_playlist)->setIconSet (QIconSet (QPixmap (normal_window_xpm)));
-    }
-    m_fullscreen = !m_fullscreen;
-    m_view->controlPanel()->popupMenu ()->setItemChecked (ControlPanel::menu_fullscreen, m_fullscreen);
-
-    if (m_fullscreen) {
-        m_mouse_invisible_timer = startTimer(MOUSE_INVISIBLE_DELAY);
-    } else {
-        if (m_mouse_invisible_timer) {
-            killTimer (m_mouse_invisible_timer);
-            m_mouse_invisible_timer = 0;
-        }
-        unsetCursor();
-    }
-}
-
-void ViewArea::minimalMode () {
-    m_minimal = !m_minimal;
-    killTimers ();
-    m_mouse_invisible_timer = m_repaint_timer = 0;
-    if (m_minimal) {
-        m_view->setViewOnly ();
-        m_view->setControlPanelMode (KMPlayer::View::CP_AutoHide);
-        m_view->setNoInfoMessages (true);
-        m_view->controlPanel ()->button (ControlPanel::button_playlist)->setIconSet (QIconSet (QPixmap (normal_window_xpm)));
-    } else {
-        m_view->setControlPanelMode (KMPlayer::View::CP_Show);
-        m_view->setNoInfoMessages (false);
-        m_view->controlPanel ()->button (ControlPanel::button_playlist)->setIconSet (QIconSet (QPixmap (playlist_xpm)));
-    }
-    m_topwindow_rect = topLevelWidget ()->geometry ();
-}
-
-KDE_NO_EXPORT void ViewArea::accelActivated () {
-    m_view->controlPanel()->popupMenu ()->activateItemAt (m_view->controlPanel()->popupMenu ()->indexOf (ControlPanel::menu_fullscreen)); 
-}
-
-KDE_NO_EXPORT void ViewArea::mousePressEvent (QMouseEvent * e) {
-    if (eventListener && eventListener->handleEvent(new PointerEvent(event_pointer_clicked,e->x(), e->y())))
-        e->accept ();
-}
-
-KDE_NO_EXPORT void ViewArea::mouseDoubleClickEvent (QMouseEvent *) {
-    m_view->fullScreen (); // screensaver stuff
-}
-
-KDE_NO_EXPORT void ViewArea::mouseMoveEvent (QMouseEvent * e) {
-    if (e->state () == Qt::NoButton) {
-        int vert_buttons_pos = height ();
-        int cp_height = m_view->controlPanel ()->maximumSize ().height ();
-        m_view->delayedShowButtons (e->y() > vert_buttons_pos-cp_height &&
-                                    e->y() < vert_buttons_pos);
-    }
-    if (eventListener && eventListener->handleEvent(new PointerEvent(event_pointer_moved,e->x(), e->y())))
-        e->accept ();
-    mouseMoved (); // for m_mouse_invisible_timer
-}
-
-KDE_NO_EXPORT void ViewArea::syncVisual (QRect rect) {
-    if (!eventListener) {
-        repaint (rect, false);
-        return;
-    }
-#define PAINT_BUFFER_HEIGHT 128
-    if (!m_paint_buffer) {
-        m_paint_buffer = new QPixmap (width (), PAINT_BUFFER_HEIGHT);
-        m_painter = new QPainter ();
-    } else if (((QPixmap *)m_paint_buffer)->width () < width ())
-        ((QPixmap *)m_paint_buffer)->resize (width (), PAINT_BUFFER_HEIGHT);
-    int py=0;
-    int ex = rect.x ();
-    int ey = rect.y ();
-    int ew = rect.width ();
-    int eh = rect.height ();
-    while (py < eh) {
-        int ph = eh-py < PAINT_BUFFER_HEIGHT ? eh-py : PAINT_BUFFER_HEIGHT;
-        m_painter->begin (m_paint_buffer);
-        m_painter->translate(-ex, -ey-py);
-        m_painter->fillRect (ex, ey+py, ew, ph, QBrush (paletteBackgroundColor ()));
-        eventListener->handleEvent(new PaintEvent(*m_painter, ex, ey+py,ew,ph));
-        m_painter->end();
-        bitBlt (this, ex, ey+py, m_paint_buffer, 0, 0, ew, ph);
-        py += PAINT_BUFFER_HEIGHT;
-    }
-    XFlush (qt_xdisplay ());
-}
-
-KDE_NO_EXPORT void ViewArea::paintEvent (QPaintEvent * pe) {
-    if (eventListener)
-        scheduleRepaint (pe->rect ().x (), pe->rect ().y (), pe->rect ().width (), pe->rect ().height ());
-    else
-        QWidget::paintEvent (pe);
-}
-
-KDE_NO_EXPORT void ViewArea::scale (int val) {
-    m_fullscreen_scale = val;
-    resizeEvent (0L);
-}
-
-KDE_NO_EXPORT void ViewArea::resizeEvent (QResizeEvent *) {
-    if (!m_view->controlPanel ()) return;
-    int x =0, y = 0;
-    int w = width ();
-    int h = height ();
-    int hsb = m_view->statusBar ()->isVisible () && !isFullScreen () ? (m_view->statusBarMode () == View::SB_Only ? h : m_view->statusBar()->maximumSize ().height ()) : 0;
-    int hcp = m_view->controlPanel ()->isVisible () ? (m_view->controlPanelMode () == View::CP_Only ? h-hsb: m_view->controlPanel()->maximumSize ().height ()) : 0;
-    int wws = w;
-    // move controlpanel over video when autohiding and playing
-    int hws = h - (m_view->controlPanelMode () == View::CP_AutoHide && m_view->widgetStack ()->visibleWidget () == m_view->viewer () ? 0 : hcp) - hsb;
-    // now scale the regions and check if video region is already sized
-    bool av_geometry_changed = false;
-    if (eventListener && wws > 0 && hws > 0) {
-        m_av_geometry = QRect (0, 0, 0, 0);
-        eventListener->handleEvent (new SizeEvent (x, y, wws, hws, m_view->keepSizeRatio () ? fit_meet : fit_fill));
-        av_geometry_changed = (m_av_geometry != QRect (0, 0, 0, 0));
-        x = m_av_geometry.x ();
-        y = m_av_geometry.y ();
-        wws = m_av_geometry.width ();
-        hws = m_av_geometry.height ();
-            //m_view->viewer ()->setAspect (region->w / region->h);
-    } else
-        m_av_geometry = QRect (x, y, wws, hws);
-
-    // finally resize controlpanel and video widget
-    if (m_view->controlPanel ()->isVisible ())
-        m_view->controlPanel ()->setGeometry (0, h-hcp-hsb, w, hcp);
-    if (m_view->statusBar ()->isVisible ())
-        m_view->statusBar ()->setGeometry (0, h-hsb, w, hsb);
-    if (m_fullscreen && wws == w && hws == h) {
-        wws = wws * m_fullscreen_scale / 100;
-        hws = hws * m_fullscreen_scale / 100;
-        x += (w - wws) / 2;
-        y += (h - hws) / 2;
-    }
-    if (!av_geometry_changed)
-        setAudioVideoGeometry (x, y, wws, hws, 0L);
-}
-
-KDE_NO_EXPORT
-void ViewArea::setAudioVideoGeometry (int x, int y, int w, int h, unsigned int * bg_color) {
-    if (m_view->controlPanelMode() == View::CP_Only) {
-        w = h = 0;
-    } else if (m_view->keepSizeRatio ()) { // scale video widget inside region
-        int hfw = m_view->viewer ()->heightForWidth (w);
-        if (hfw > 0)
-            if (hfw > h) {
-                int old_w = w;
-                w = int ((1.0 * h * w)/(1.0 * hfw));
-                x += (old_w - w) / 2;
-            } else {
-                y += (h - hfw) / 2;
-                h = hfw;
-            }
-    }
-    m_av_geometry = QRect (x, y, w, h);
-    QRect rect = m_view->widgetStack ()->geometry ();
-    if (m_av_geometry != rect) {
-        m_view->widgetStack ()->setGeometry (x, y, w, h);
-        rect.unite (m_av_geometry);
-        scheduleRepaint (rect.x (), rect.y (), rect.width (), rect.height ());
-    }
-    if (bg_color)
-        if (QColor (QRgb (*bg_color)) != (m_view->viewer ()->paletteBackgroundColor ())) {
-            m_view->viewer()->setCurrentBackgroundColor (QColor (QRgb (*bg_color)));
-            scheduleRepaint (x, y, w, h);
-        }
-}
-
-KDE_NO_EXPORT void ViewArea::setEventListener (NodePtr el) {
-    if (eventListener != el) {
-        eventListener = el;
-        resizeEvent (0L);
-        if (m_repaint_timer) {
-            killTimer (m_repaint_timer);
-            m_repaint_timer = 0;
-        }
-        m_view->viewer()->resetBackgroundColor ();
-        repaint ();
-    }
-}
-
-KDE_NO_EXPORT void ViewArea::showEvent (QShowEvent *) {
-    resizeEvent (0L);
-}
-
-KDE_NO_EXPORT void ViewArea::dropEvent (QDropEvent * de) {
-    m_view->dropEvent (de);
-}
-
-KDE_NO_EXPORT void ViewArea::dragEnterEvent (QDragEnterEvent* dee) {
-    m_view->dragEnterEvent (dee);
-}
-
-KDE_NO_EXPORT void ViewArea::contextMenuEvent (QContextMenuEvent * e) {
-    m_view->controlPanel ()->popupMenu ()->exec (e->globalPos ());
-}
-
-KDE_NO_EXPORT void ViewArea::mouseMoved () {
-    if (m_fullscreen) {
-        if (m_mouse_invisible_timer)
-            killTimer (m_mouse_invisible_timer);
-        unsetCursor ();
-        m_mouse_invisible_timer = startTimer (MOUSE_INVISIBLE_DELAY);
-    }
-}
-
-KDE_NO_EXPORT void ViewArea::scheduleRepaint (int x, int y, int w, int h) {
-    if (m_repaint_timer)
-        m_repaint_rect = m_repaint_rect.unite (QRect (x, y, w, h));
-    else {
-        m_repaint_rect = QRect (x, y, w, h);
-        m_repaint_timer = startTimer (10); // 100 per sec should do
-    }
-}
-
-KDE_NO_EXPORT
-void ViewArea::moveRect (int x, int y, int w, int h, int x1, int y1) {
-    QRect r (x, y, w, h);
-    if (m_repaint_timer && m_repaint_rect.intersects (r)) {
-        m_repaint_rect = m_repaint_rect.unite (QRect (x1, y1, w, h).unite (r));
-    } else if (m_view->viewer()->frameGeometry ().intersects (r)) {
-        QRect r2 (QRect (x1, y1, w, h).unite (r));
-        scheduleRepaint (r.x (), r.y (), r.width (), r.height ());
-    } else {
-        bitBlt (this, x1, y1, this, x, y, w, h);
-        if (x1 > x)
-            syncVisual (QRect (x, y, x1 - x, h));
-        else if (x > x1)
-            syncVisual (QRect (x1 + w, y, x - x1, h));
-        if (y1 > y)
-            syncVisual (QRect (x, y, w, y1 - y));
-        else if (y > y1)
-            syncVisual (QRect (x, y1 + h, w, y - y1));
-    }
-}
-
-KDE_NO_EXPORT void ViewArea::timerEvent (QTimerEvent * e) {
-    if (e->timerId () == m_mouse_invisible_timer) {
-        killTimer (m_mouse_invisible_timer);
-        m_mouse_invisible_timer = 0;
-        if (m_fullscreen)
-            setCursor (BlankCursor);
-    } else if (e->timerId () == m_repaint_timer) {
-        killTimer (m_repaint_timer);
-        m_repaint_timer = 0;
-        //repaint (m_repaint_rect, false);
-        syncVisual (m_repaint_rect);
-    } else {
-        kdError () << "unknown timer " << e->timerId () << " " << m_repaint_timer << endl;
-        killTimer (e->timerId ());
-    }
-}
-
-KDE_NO_EXPORT void ViewArea::closeEvent (QCloseEvent * e) {
-    //kdDebug () << "closeEvent" << endl;
-    if (m_fullscreen) {
-        fullScreen ();
-        if (!m_parent->topLevelWidget ()->isVisible ())
-            m_parent->topLevelWidget ()->show ();
-        e->ignore ();
-    } else
-        QWidget::closeEvent (e);
-}
-
-//-----------------------------------------------------------------------------
-
 namespace KMPlayer {
 
 class KMPlayerPictureWidget : public QWidget {
@@ -460,6 +140,7 @@ KDE_NO_CDTOR_EXPORT View::View (QWidget *parent, const char *name)
     m_old_controlpanel_mode (CP_Show),
     m_statusbar_mode (SB_Hide),
     controlbar_timer (0),
+    infopanel_timer (0),
     m_keepsizeratio (false),
     m_playing (false),
     m_mixer_init (false),
@@ -494,7 +175,8 @@ KDE_NO_EXPORT void View::dragEnterEvent (QDragEnterEvent* dee) {
 }
 
 KDE_NO_EXPORT void View::init (KActionCollection * action_collection) {
-    //setBackgroundMode(Qt::NoBackground);
+    setBackgroundMode(Qt::NoBackground); // prevents flashing
+    //m_dockarea->setEraseColor (QColor (0, 0, 0));
     QPalette pal (QColor (64, 64,64), QColor (32, 32, 32));
     QVBoxLayout * viewbox = new QVBoxLayout (this, 0, 0);
     m_dockarea = new KDockArea (this, "kde_kmplayer_dock_area");
@@ -576,8 +258,8 @@ KDE_NO_EXPORT void View::setEraseColor (const QColor & color) {
 void View::setInfoMessage (const QString & msg) {
     bool ismain = m_dockarea->getMainDockWidget () == m_dock_infopanel;
     if (msg.isEmpty ()) {
-        if (!ismain && !m_edit_mode)
-            m_dock_infopanel->undock ();
+        if (!ismain && !m_edit_mode && !infopanel_timer)
+            infopanel_timer = startTimer (0);
        m_infopanel->clear ();
     } else if (ismain || !m_no_info) {
         if (!m_edit_mode && m_dock_infopanel->mayBeShow ())
@@ -605,7 +287,7 @@ void View::toggleShowPlaylist () {
             h +=style.pixelMetric(QStyle::PM_DockWindowHandleExtent,m_playlist);
             for (QListViewItem *i=m_playlist->firstChild();i;i=i->itemBelow()) {
                 h += i->height ();
-                if (h > int (0.5 * height ())) {
+                if (h > int (0.25 * height ())) {
                     horz = false;
                     break;
                 }
@@ -714,8 +396,12 @@ KDE_NO_EXPORT void View::updateVolume () {
 
 void View::showWidget (WidgetType wt) {
     m_widgetstack->raiseWidget (m_widgettypes [wt]);
-    if (m_widgetstack->visibleWidget () == m_widgettypes[WT_Console])
+    if (m_widgetstack->visibleWidget () == m_widgettypes[WT_Console]) {
         addText (QString (""), false);
+        if (m_controlpanel_mode == CP_AutoHide && m_playing)
+            m_control_panel->show();
+    } else
+        delayedShowButtons (false);
     updateLayout ();
 }
 
@@ -736,18 +422,23 @@ void View::setControlPanelMode (ControlPanelMode m) {
     m_old_controlpanel_mode = m_controlpanel_mode = m;
     if (m_playing && isFullScreen())
         m_controlpanel_mode = CP_AutoHide;
-    if (m_control_panel)
-        if (m_controlpanel_mode == CP_Show || m_controlpanel_mode == CP_Only)
+    if ((m_controlpanel_mode == CP_Show || m_controlpanel_mode == CP_Only) &&
+            !m_control_panel->isVisible ()) {
+        m_control_panel->show ();
+        m_view_area->resizeEvent (0L);
+    } else if (m_controlpanel_mode == CP_AutoHide) { 
+        if ((m_playing &&
+                m_widgetstack->visibleWidget () == m_widgettypes[WT_Video]) ||
+                m_widgetstack->visibleWidget () == m_widgettypes[WT_Picture])
+            delayedShowButtons (false);
+        else if (!m_control_panel->isVisible ()) {
             m_control_panel->show ();
-        else if (m_controlpanel_mode == CP_AutoHide) { 
-            if (m_playing || m_widgetstack->visibleWidget () == m_widgettypes[WT_Picture])
-                delayedShowButtons (false);
-            else
-                m_control_panel->show ();
-        } else
-            m_control_panel->hide ();
-    //m_view_area->setMouseTracking (m_controlpanel_mode == CP_AutoHide && m_playing);
-    m_view_area->resizeEvent (0L);
+            m_view_area->resizeEvent (0L);
+        }
+    } else if (m_controlpanel_mode == CP_Hide && m_control_panel->isVisible()) {
+        m_control_panel->hide ();
+        m_view_area->resizeEvent (0L);
+    }
 }
 
 void View::setStatusBarMode (StatusBarMode m) {
@@ -760,12 +451,18 @@ void View::setStatusBarMode (StatusBarMode m) {
 }
 
 KDE_NO_EXPORT void View::delayedShowButtons (bool show) {
-    if (m_controlpanel_mode != CP_AutoHide || controlbar_timer ||
-        (m_control_panel &&
-         (show && m_control_panel->isVisible ()) || 
-         (!show && !m_control_panel->isVisible ())))
-        return;
-    controlbar_timer = startTimer (500);
+    if ((show && m_control_panel->isVisible ()) || 
+            (!show && !m_control_panel->isVisible ())) {
+        if (controlbar_timer) {
+            killTimer (controlbar_timer);
+            controlbar_timer = 0;
+        }
+    } else if (m_controlpanel_mode == CP_AutoHide &&
+            (m_playing ||
+             m_widgetstack->visibleWidget () == m_widgettypes[WT_Picture]) &&
+            m_widgetstack->visibleWidget () != m_widgettypes[WT_Console] &&
+            !controlbar_timer)
+        controlbar_timer = startTimer (500);
 }
 
 KDE_NO_EXPORT void View::setVolume (int vol) {
@@ -794,22 +491,30 @@ void View::setKeepSizeRatio (bool b) {
 KDE_NO_EXPORT void View::timerEvent (QTimerEvent * e) {
     if (e->timerId () == controlbar_timer) {
         controlbar_timer = 0;
-        if (!m_playing && m_widgetstack->visibleWidget () != m_widgettypes[WT_Picture])
-            return;
-        int vert_buttons_pos = m_view_area->height ();
-        int mouse_pos = m_view_area->mapFromGlobal (QCursor::pos ()).y();
-        int cp_height = m_control_panel->maximumSize ().height ();
-        bool mouse_on_buttons = (//m_view_area->hasMouse () && 
-                mouse_pos >= vert_buttons_pos-cp_height &&
-                mouse_pos <= vert_buttons_pos);
-        if (m_control_panel)
-            if (mouse_on_buttons && !m_control_panel->isVisible ())
+        if (m_playing ||
+                m_widgetstack->visibleWidget () == m_widgettypes[WT_Picture]) {
+            int vert_buttons_pos = m_view_area->height()-statusBarHeight ();
+            QPoint mouse_pos = m_view_area->mapFromGlobal (QCursor::pos ());
+            int cp_height = m_control_panel->maximumSize ().height ();
+            bool mouse_on_buttons = (//m_view_area->hasMouse () && 
+                    mouse_pos.y () >= vert_buttons_pos-cp_height &&
+                    mouse_pos.y ()<= vert_buttons_pos &&
+                    mouse_pos.x () > 0 &&
+                    mouse_pos.x () < m_control_panel->width());
+            if (mouse_on_buttons && !m_control_panel->isVisible ()) {
                 m_control_panel->show ();
-            else if (!mouse_on_buttons && m_control_panel->isVisible ())
+                m_view_area->resizeEvent (0L);
+            } else if (!mouse_on_buttons && m_control_panel->isVisible ()) {
                 m_control_panel->hide ();
+                m_view_area->resizeEvent (0L);
+            }
+        }
+    } else if (e->timerId () == infopanel_timer) {
+        if (m_infopanel->text ().isEmpty ())
+            m_dock_infopanel->undock ();
+        infopanel_timer  = 0;
     }
     killTimer (e->timerId ());
-    m_view_area->resizeEvent (0L);
 }
 
 void View::addText (const QString & str, bool eol) {
@@ -875,18 +580,20 @@ KDE_NO_EXPORT void View::playingStart () {
 }
 
 KDE_NO_EXPORT void View::playingStop () {
-    if (m_control_panel && m_controlpanel_mode == CP_AutoHide) {
+    if (m_controlpanel_mode == CP_AutoHide &&
+            m_widgetstack->visibleWidget () != m_widgettypes[WT_Picture]) {
         m_control_panel->show ();
         //m_view_area->setMouseTracking (false);
     }
+    killTimer (controlbar_timer);
+    controlbar_timer = 0;
     m_playing = false;
     XClearWindow (qt_xdisplay(), m_viewer->embeddedWinId ());
     m_view_area->resizeEvent (0L);
 }
 
 KDE_NO_EXPORT void View::leaveEvent (QEvent *) {
-    if (m_controlpanel_mode == CP_AutoHide && m_playing)
-        delayedShowButtons (false);
+    delayedShowButtons (false);
 }
 
 KDE_NO_EXPORT void View::reset () {
@@ -931,6 +638,16 @@ void View::fullScreen () {
     emit fullScreenChanged ();
 }
 
+KDE_NO_EXPORT int View::statusBarHeight () const {
+    if (statusBar()->isVisible () && !viewArea()->isFullScreen ()) {
+        if (statusBarMode () == SB_Only)
+            return height ();
+        else 
+            return statusBar()->maximumSize ().height ();
+    }
+    return 0;
+}
+
 KDE_NO_EXPORT bool View::x11Event (XEvent * e) {
     switch (e->type) {
         case UnmapNotify:
@@ -956,9 +673,10 @@ KDE_NO_EXPORT bool View::x11Event (XEvent * e) {
             fprintf (stderr, "colormap notify\n");
             return true;*/
         case MotionNotify:
-            if (m_playing && e->xmotion.window == m_viewer->embeddedWinId ())
+            if (e->xmotion.window == m_viewer->embeddedWinId ())
                 delayedShowButtons (e->xmotion.y > m_view_area->height () -
-                                    m_control_panel->maximumSize ().height ());
+                        statusBarHeight () -
+                        m_control_panel->maximumSize ().height ());
             m_view_area->mouseMoved ();
             break;
         case MapNotify:
